@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
-# $Revision: 1.23 $ 
-# $Date: 2005/03/29 21:07:27 $
+# $Revision: 1.33 $
+# $Date: 2005/08/18 17:48:54 $
 # $Author: dwelch $
 #
 # (c) Copyright 2003-2004 Hewlett-Packard Development Company, L.P.
@@ -74,6 +74,8 @@ ERROR_SMTP_SENDER_REFUSED = 43
 ERROR_SMTP_DATA_ERROR = 44
 ERROR_INVALID_HOSTNAME = 45
 ERROR_INVALID_PORT_NUMBER = 46
+ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE = 47
+ERROR_UNSUPPORTED_MODEL = 48
 ERROR_UNABLE_TO_BIND_SOCKET = 95
 # --> add new codes here <--
 ERROR_UNABLE_TO_CONTACT_SERVICE = 96
@@ -88,7 +90,7 @@ ERROR_INTERNAL = 99
 
 
 
-# Event and status codes 
+# Event and status codes
 # These are used for the 'status-code' returned by DeviceQuery (STATUS_*)
 # and by the event-code used by Event (EVENT_* + STATUS_*)
 
@@ -178,21 +180,29 @@ STATUS_PRINTER_OFFLINE = 1803
 STATUS_PRINTER_LOW_TONER = 1804
 STATUS_PRINTER_NO_TONER = 1805
 STATUS_PRINTER_SERVICE_REQUEST = 1806
+#end
+
+# other "synthetic" codes
+STATUS_DEVICE_UNSUPPORTED = 1900
+#end
 
 # scan
 EVENT_START_SCAN_JOB = 2000
 EVENT_END_SCAN_JOB = 2001
 EVENT_SCANNER_FAIL = 2002
+#end
 
 # fax
 EVENT_START_FAX_JOB = 3000
 EVENT_END_FAX_JOB = 3001
 EVENT_FAX_JOB_FAIL = 3002
+#end
 
 # copy
 EVENT_START_COPY_JOB = 4000
 EVENT_END_COPY_JOB = 4001
 EVENT_COPY_JOB_FAIL = 4002
+#end
 
 # Adding the ERROR_CODE_BASE to the above
 # ERROR codes will produce a event/status code
@@ -251,6 +261,12 @@ EVENT_UI_SHOW_TOOLBOX = 7000
 EVENT_UI_HIDE_TOOLBOX = 7001
 # end
 
+# fax
+EVENT_FAX_MIN = 8000
+EVENT_FAX_RENDERED = 8000 # sent by hpfax: after job rendered and spooled
+EVENT_FAX_RECEIPTS_SET = 8001 # sent by sendfax after receips and options gathered
+EVENT_FAX_MAX = 8999
+# end
 
 # Error states
 ERROR_STATE_CLEAR = 0        # Show icon w/o overlay
@@ -259,7 +275,7 @@ ERROR_STATE_WARNING = 2      # Icon w/ triangle "!" overlay
 ERROR_STATE_ERROR = 3        # Icon w/ circle "X" overlay
 ERROR_STATE_LOW_SUPPLIES = 4 # Icon w/ supplies overlay
 ERROR_STATE_BUSY = 5         # Icon with busy overlay
-
+#end
 
 
 # Map of status/event codes to UI states
@@ -278,7 +294,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     STATUS_PRINTER_PEN_CHANGE : ERROR_STATE_WARNING,
     STATUS_PRINTER_OUT_OF_PAPER : ERROR_STATE_ERROR,
     STATUS_PRINTER_BANNER_EJECT : ERROR_STATE_ERROR,
-    STATUS_PRINTER_BANNER_MISMATCH : ERROR_STATE_WARNING, 
+    STATUS_PRINTER_BANNER_MISMATCH : ERROR_STATE_WARNING,
     STATUS_PRINTER_PHOTO_MISMATCH : ERROR_STATE_WARNING,
     STATUS_PRINTER_DUPLEX_MISMATCH : ERROR_STATE_WARNING,
     STATUS_PRINTER_MEDIA_JAM : ERROR_STATE_ERROR,
@@ -320,6 +336,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     STATUS_PRINTER_LOW_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_NO_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_SERVICE_REQUEST : ERROR_STATE_ERROR,
+    STATUS_DEVICE_UNSUPPORTED : ERROR_STATE_ERROR,
     # The following block are EVENTs because they are only
     # recieved as events from hpiod, hp backend, etc.
     # i.e., a device does not produce status codes in this range
@@ -388,8 +405,7 @@ DEVICE_STATE_JUST_FOUND = 1
 IO_STATE_HP_OPEN = 0
 IO_STATE_HP_READY = 1
 IO_STATE_HP_NOT_AVAIL = 2
-IO_STATE_NON_HP = 3   
-
+IO_STATE_NON_HP = 3
 
 #
 # Defines for model query types and status query
@@ -408,7 +424,7 @@ AGENT_KIND_ADF_KIT = 6 # LaserJet "Document feeder kit"
 AGENT_KIND_DRUM_KIT = 7 # LaserJet
 AGENT_KIND_TRANSFER_KIT = 8 # LaserJet
 AGENT_KIND_INT_BATTERY = 9 # Mobile deskjet (DJ450, etc)
-AGENT_KIND_UNKNOWN = 0x3e
+AGENT_KIND_UNKNOWN = 0x3e # (62)
 
 # 'type'
 AGENT_TYPE_NONE = 0
@@ -425,8 +441,8 @@ AGENT_TYPE_GGK = 10
 AGENT_TYPE_BLUE = 11
 AGENT_TYPE_WHITE = 0x20 # For ISO 10180 compatibility
 AGENT_TYPE_RED = 0x21 # For ISO 10180 compatibility
-AGENT_TYPE_UNSPECIFIED = 0x3e # Used for kind = 5, 6, 7, 8, or 9
-AGENT_TYPE_ERROR = 0x3f
+AGENT_TYPE_UNSPECIFIED = 0x3e # (62) Used for kind = 5, 6, 7, 8, or 9
+AGENT_TYPE_ERROR = 0x3f # (63)
 
 # 'health'
 AGENT_HEALTH_OK = 0
@@ -453,8 +469,10 @@ AGENT_CONFIG_NONE = 0
 AGENT_CONFIG_BLACK_ONLY = 1
 AGENT_CONFIG_PHOTO_ONLY = 2
 AGENT_CONFIG_COLOR_ONLY = 3
-AGENT_CONFIG_COLOR_AND_BLACK = 4
-AGENT_CONFIG_COLOR_AND_PHOTO = 5
+AGENT_CONFIG_GREY_ONLY = 4
+AGENT_CONFIG_COLOR_AND_BLACK = 5
+AGENT_CONFIG_COLOR_AND_PHOTO = 6
+AGENT_CONFIG_COLOR_AND_GREY = 7
 AGENT_CONFIG_INVALID = 99
 
 # align-types
@@ -468,6 +486,7 @@ ALIGN_TYPE_LIDIL_AIO = 6
 ALIGN_TYPE_LIDIL_VIP = 7
 ALIGN_TYPE_DESKJET_450 = 8
 ALIGN_TYPE_9XX_NO_EDGE_ALIGN = 9
+ALIGN_TYPE_LBOW = 10
 
 # clean-types
 CLEAN_TYPE_NONE = 0
@@ -480,10 +499,11 @@ COLOR_CAL_TYPE_NONE = 0
 COLOR_CAL_TYPE_DESKJET_450 = 1
 COLOR_CAL_TYPE_MALIBU_CRICK = 2
 COLOR_CAL_TYPE_STRINGRAY_LONGBOW_TORNADO = 3
+COLOR_CAL_TYPE_CONNERY = 4
 
 # status-types
 STATUS_TYPE_NONE = 0
-STATUS_TYPE_VSTATUS = 1 
+STATUS_TYPE_VSTATUS = 1
 STATUS_TYPE_S = 2
 STATUS_TYPE_LJ = 3
 STATUS_TYPE_S_W_BATTERY = 4
@@ -546,3 +566,25 @@ IN_TRAY_NOT_PRESENT = 0
 IN_TRAY_PRESENT = 1 # for !S:02, test for > IN_TRAY_NOT_PRESENT
 IN_TRAY_DEFAULT = 2 # S:02 only
 IN_TRAY_LOCKED = 3 # S:02 only
+
+# 'io-support'
+IO_SUPPORT_PARALLEL = 0x1
+IO_SUPPORT_USB = 0x2
+IO_SUPPORT_NETWORK = 0x4
+IO_SUPPORT_WIRELESS = 0x8
+IO_SUPPORT_BLUETOOTH = 0x10
+
+# Model categories
+MODEL_TYPE_COLOR_INKJET = 0
+MODEL_TYPE_PHOTOSMART = 1
+MODEL_TYPE_COLOR_LASER = 2
+MODEL_TYPE_COLOR_LASER_AIO = 3
+MODEL_TYPE_COLOR_INKJET_AIO = 4
+MODEL_TYPE_MONO_LASER = 5
+MODEL_TYPE_MONO_LASER_AIO = 6
+
+# 'io-mode'
+IO_MODE_UNI = 0
+IO_MODE_RAW = 1
+IO_MODE_MLC = 2
+IO_MODE_DOT4 = 3
