@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# $Revision: 1.46 $ 
-# $Date: 2005/03/23 23:54:06 $
+# $Revision: 1.62 $
+# $Date: 2005/09/08 18:23:07 $
 # $Author: dwelch $
 #
 # (c) Copyright 2001-2005 Hewlett-Packard Development Company, L.P.
@@ -26,22 +26,11 @@
 # Thanks to Henrique M. Holschuh <hmh@debian.org> for various security patches
 #
 
-from __future__ import generators 
+from __future__ import generators
 
 # Std Lib
-import sys, os
-import fnmatch
-import tempfile
-import socket
-import msg
-import struct
-import select
-import time
-import traceback
-import fcntl
-import errno
-import stat
-import string
+import sys, os, fnmatch, tempfile, socket, struct, select, time
+import fcntl, errno, stat, string, xml.parsers.expat, commands
 
 # Local
 from g import *
@@ -50,17 +39,17 @@ from codes import *
 
 def Translator(frm='', to='', delete='', keep=None):
     allchars = string.maketrans('','')
-    
+
     if len(to) == 1:
         to = to * len(frm)
     trans = string.maketrans(frm, to)
-    
+
     if keep is not None:
         delete = allchars.translate(allchars, keep.translate(allchars, delete))
-    
+
     def callable(s):
         return s.translate(trans, delete)
-    
+
     return callable
 
 # For pidfile locking (must be "static" and global to the whole app)
@@ -70,7 +59,7 @@ prv_pidfile_name = ""
 
 def get_pidfile_lock ( a_pidfile_name="" ):
     """ Call this to either lock the pidfile, or to update it after a fork()
-        Credit: Henrique M. Holschuh <hmh@debian.org> 
+        Credit: Henrique M. Holschuh <hmh@debian.org>
     """
     global prv_pidfile
     global prv_pidfile_name
@@ -119,28 +108,28 @@ def daemonize ( stdin='/dev/null', stdout='/dev/null', stderr='/dev/null' ):
     """
     # Try to lock pidfile if not locked already
     if prv_pidfile_name != '' or prv_pidfile_name != "":
-        get_pidfile_lock( prv_pidfile_name )    
+        get_pidfile_lock( prv_pidfile_name )
 
     # Do first fork.
-    try: 
-        pid = os.fork() 
+    try:
+        pid = os.fork()
         if pid > 0:
             sys.exit(0) # Exit first parent.
-    except OSError, e: 
+    except OSError, e:
         sys.stderr.write ("fork #1 failed: (%d) %s\n" % (e.errno, e.strerror)    )
         sys.exit(1)
 
     # Decouple from parent environment.
-    os.chdir("/") 
-    os.umask(0) 
-    os.setsid() 
+    os.chdir("/")
+    os.umask(0)
+    os.setsid()
 
     # Do second fork.
-    try: 
-        pid = os.fork() 
+    try:
+        pid = os.fork()
         if pid > 0:
             sys.exit(0) # Exit second parent.
-    except OSError, e: 
+    except OSError, e:
         sys.stderr.write ("fork #2 failed: (%d) %s\n" % (e.errno, e.strerror)    )
         sys.exit(1)
 
@@ -164,7 +153,7 @@ def ifelse( cond, t, f ):
     else: return f
 
 def to_bool_str( s, default='0' ):
-    """ Convert an arbitrary 0/1/T/F/Y/N string to a normalized string 0/1.""" 
+    """ Convert an arbitrary 0/1/T/F/Y/N string to a normalized string 0/1."""
     if len( s ):
         if s[0].lower() in [ '1', 't', 'y' ]:
             return '1'
@@ -174,7 +163,7 @@ def to_bool_str( s, default='0' ):
     return default
 
 def to_bool( s, default=False ):
-    """ Convert an arbitrary 0/1/T/F/Y/N string to a boolean True/False value.""" 
+    """ Convert an arbitrary 0/1/T/F/Y/N string to a boolean True/False value."""
     if len( s ):
         if s[0].lower() in [ '1', 't', 'y' ]:
             return True
@@ -230,20 +219,20 @@ def walkFiles( root, recurse=True, abs_paths=False, return_folders=False, patter
 
 
 def is_path_writable( path ):
-    
+
     if os.path.exists( path ):
         s = os.stat( path )
         mode = s[ stat.ST_MODE ] & 0777
-        
+
         if mode & 02:
             return True
         elif s[ stat.ST_GID ] == os.getgid() and mode & 020:
             return True
         elif s[ stat.ST_UID ] == os.getuid() and mode & 0200:
             return True
-    
+
     return False
-    
+
 
 # Provides the TextFormatter class for formatting text into columns.
 # Original Author: Hamish B Lawson, 1999
@@ -257,7 +246,7 @@ class TextFormatter:
     def __init__( self, colspeclist ):
         self.columns = []
         for colspec in colspeclist:
-            self.columns.append( Column( **colspec ) ) 
+            self.columns.append( Column( **colspec ) )
 
     def compose(self, textlist, add_newline=False):
         numlines = 0
@@ -327,114 +316,6 @@ class Column:
 
 
 
-def getInteractiveDeviceURI( bus='cups,usb', filter='none' ): 
-    from prnt import cups
-    bus = bus.lower()
-
-    log.warn( "Device not specified.")
-
-##    if bus == 'cups':
-##        log.info(  bold( "\nChoose device:" ) )
-##        cups_devices = {}
-##        cups_printers = cups.getPrinters()
-##        x = len( cups_printers )
-##        max_deviceid_size = 0
-##
-##        for p in cups_printers:
-##            device_uri = p.device_uri
-##            if p.device_uri == '':
-##                device_uri = '?UNKNOWN?'
-##
-##            try:
-##                cups_devices[ device_uri ]
-##            except KeyError: 
-##                cups_devices[ device_uri ] = ( device_uri, [ p.name ] )
-##            else:
-##                cups_devices[ device_uri ][1].append( p.name )
-##
-##            max_deviceid_size = max( len( p.device_uri ), max_deviceid_size ) 
-##
-##        devices, x = {}, 0
-##        for y in cups_devices:
-##            devices[x] = cups_devices[y]
-##            x += 1
-
-    #else:   
-    if 1:
-        log.info(  bold( "\nChoose device from probed devices connected on bus(es) %s:\n" % bus ) ) 
-        hpssd_sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
-        hpssd_sock.connect( ( prop.hpssd_host, prop.hpssd_port ) )
-
-        fields, data = msg.xmitMessage( hpssd_sock, 
-                                        "ProbeDevicesFiltered",
-                                        None, 
-                                        { 
-                                            'bus' : bus,
-                                            'timeout' : 5,
-                                            'ttl' : 4,
-                                            'format' : 'default',
-                                            'filter' : filter,
-
-                                        } 
-                                      )
-        hpssd_sock.close()  
-        temp = data.splitlines()
-        probed_devices = []
-        for t in temp:
-            probed_devices.append( t.split(',')[0] )
-        cups_printers = cups.getPrinters()
-        log.debug( probed_devices )
-        log.debug( cups_printers )
-        max_deviceid_size, x, devices = 0, 0, {} 
-
-        for d in probed_devices:
-            printers = []
-            for p in cups_printers:
-                if p.device_uri == d:
-                    printers.append( p.name )
-            devices[x] = ( d, printers )
-            x += 1
-            max_deviceid_size = max( len(d), max_deviceid_size )
-
-    if x == 0:
-        log.error( "No devices found." )
-        raise Error( ERROR_NO_PROBED_DEVICES_FOUND )
-    elif x == 1:
-        log.info( "Using device: %s" % devices[0][0] )
-        return devices[0][0]
-    else:    
-
-        formatter = TextFormatter( 
-                (
-                    {'width': 4},
-                    {'width': max_deviceid_size, 'margin': 2},
-                    {'width': 80-max_deviceid_size-8, 'margin': 2},
-                )
-            )
-        log.info(  formatter.compose( ( "Num.", "Device-URI", "CUPS printer(s)" ) ) )
-        log.info( formatter.compose( ( '-'*4, '-'*(max_deviceid_size), '-'*( 80-max_deviceid_size-10 ) ) ) )
-        for y in range( x ):
-            log.info(  formatter.compose( ( str(y), devices[y][0], ', '.join( devices[y][1] ) ) ) )
-
-        while 1:
-            user_input = raw_input( bold( "\nEnter number 0...%d for device (q=quit) ?" % (x-1) ) )
-            if user_input == '': 
-                log.warn( "Invalid input - enter a numeric value or 'q' to quit." )
-                continue
-            if user_input.strip()[0] in ( 'q', 'Q' ):
-                return
-            try:
-                i = int(user_input)
-            except ValueError:
-                log.warn( "Invalid input - enter a numeric value or 'q' to quit." )
-                continue
-            if i < 0 or i > (x-1):
-                log.warn( "Invalid input - enter a value between 0 and %d or 'q' to quit." % (x-1) )
-                continue
-            break
-
-        return devices[i][0]
-
 
 
 class Stack:
@@ -454,7 +335,7 @@ class Stack:
         self.stack = []
 
 
-# RingBuffer class       
+# RingBuffer class
 # Source: Python Cookbook 1st Ed., sec. 5.18, pg. 201
 # Credit: Sebastien Keim
 # License: Modified BSD
@@ -477,7 +358,7 @@ class RingBufferFull:
     def __init__(self,n):
         #raise "you should use RingBuffer"
         pass
-    def append(self,x):     
+    def append(self,x):
         self.data[self.cur]=x
         self.cur=(self.cur+1) % self.max
     def get(self):
@@ -521,7 +402,7 @@ def updateCRC(crc, data, mask=MASK_CRC16):
                 crc = crc << 1L
             c = c << 1L
 
-    return crc & 0xffffL    
+    return crc & 0xffffL
 
 def calcCRC( data ):
     crc = 0
@@ -619,7 +500,7 @@ def red(text):
     return codes["red"]+text+codes["reset"]
 
 def darkred(text):
-    return codes["darkred"]+text+codes["reset"]    
+    return codes["darkred"]+text+codes["reset"]
 
 
 def commafy(val):
@@ -646,19 +527,19 @@ def format_bytes( s, show_bytes=False ):
 
 try:
     make_temp_file = tempfile.mkstemp # 2.3+
-except AttributeError: 
+except AttributeError:
     def make_temp_file( suffix='', prefix='', dir='', text=False ): # pre-2.3
-        path = tempfile.mktemp( suffix ) 
+        path = tempfile.mktemp( suffix )
         fd = os.open( path, os.O_RDWR|os.O_CREAT|os.O_EXCL, 0700 )
         #os.unlink( path ) # TODO... make this secure
-        #return ( os.fdopen( fd, 'w+b' ), path ) 
+        #return ( os.fdopen( fd, 'w+b' ), path )
         return ( fd, path )
 
 def log_title( program_name, version ):
     log.info( "" )
     log.info( bold( "HP Linux Imaging and Printing System (ver. %s)" % prop.version ) )
     log.info( bold( "%s ver. %s" % ( program_name,version) ) )
-    log.info( "" ) 
+    log.info( "" )
     log.info( "Copyright (c) 2003-5 Hewlett-Packard Development Company, LP" )
     log.info( "This software comes with ABSOLUTELY NO WARRANTY." )
     log.info( "This is free software, and you are welcome to distribute it" )
@@ -685,38 +566,55 @@ def which( command ):
 def deviceDefaultFunctions():
     cmd_print, cmd_copy, cmd_fax, cmd_pcard, cmd_scan = \
         '', '', '', '', ''
-    
+
     # Print
-    path = which( 'kprinter' )
-    
-    if len(path) > 0:
-        cmd_print = 'kprinter -P%PRINTER% --system cups'
+    path = which( 'hp-print' )
+
+    if len( path ) > 0:
+        cmd_print = 'hp-print -p%PRINTER%'
     else:
-        path = which( 'gtklp' )
+        path = which( 'kprinter' )
 
         if len(path) > 0:
-            cmd_print = 'gtklp -P%PRINTER%'
-        
+            cmd_print = 'kprinter -P%PRINTER% --system cups'
         else:
-            path = which( 'xpp' )
-             
-            if len( path ) > 0:
-                cmd_print = 'xpp -P%PRINTER%'
-            
+            path = which( 'gtklp' )
+
+            if len(path) > 0:
+                cmd_print = 'gtklp -P%PRINTER%'
+
+            else:
+                path = which( 'xpp' )
+
+                if len( path ) > 0:
+                    cmd_print = 'xpp -P%PRINTER%'
+
 
     # Scan
     path = which( 'xsane' )
 
     if len(path) > 0:
         cmd_scan = 'xsane -V %SANE_URI%'
+    else:
+        path = which( 'kooka' )
+
+        if len(path)>0:
+            #cmd_scan = 'kooka -d "%SANE_URI%"'
+            cmd_scan = 'kooka'
+
+        else:
+            path = which( 'xscanimage' )
+
+            if len(path)>0:
+                cmd_scan = 'xscanimage'
 
     # Photo Card
-    files = os.listdir( prop.home_dir )
-    
-    if 'unload' in files:
-        cmd_pcard = 'python %HOME%/unload -d %DEVICE_URI%'
-    
-    elif 'unload.py' in files:
+    path = which( 'hp-unload' )
+
+    if len( path ) > 0:
+        cmd_pcard = 'hp-unload -d %DEVICE_URI%'
+
+    else:
         cmd_pcard = 'python %HOME%/unload.py -d %DEVICE_URI%'
 
     # Copy
@@ -770,125 +668,469 @@ ICMP_ECHO_REQUEST = 8 # Seems to be the same on Solaris.
 # I'm not too confident that this is right but testing seems
 # to suggest that it gives the same answers as in_cksum in ping.c
 def checksum( str ):
-  csum = 0
-  countTo = ( len(str) / 2 ) * 2
-  count = 0
-  while count < countTo:
-    thisVal = ord( str[count+1] ) * 256 + ord( str[count] )
-    csum = csum + thisVal
-    csum = csum & 0xffffffffL # Necessary?
-    count = count + 2
+    csum = 0
+    countTo = ( len(str) / 2 ) * 2
+    count = 0
+    while count < countTo:
+        thisVal = ord( str[count+1] ) * 256 + ord( str[count] )
+        csum = csum + thisVal
+        csum = csum & 0xffffffffL # Necessary?
+        count = count + 2
 
-  if countTo < len(str):
-    csum = csum + ord( str[ len(str) - 1 ] )
-    csum = csum & 0xffffffffL # Necessary?
+    if countTo < len(str):
+        csum = csum + ord( str[ len(str) - 1 ] )
+        csum = csum & 0xffffffffL # Necessary?
 
-  csum = ( csum >> 16 ) + ( csum & 0xffff )
-  csum = csum + ( csum >> 16 )
-  answer = ~csum
-  answer = answer & 0xffff
+    csum = ( csum >> 16 ) + ( csum & 0xffff )
+    csum = csum + ( csum >> 16 )
+    answer = ~csum
+    answer = answer & 0xffff
 
-  # Swap bytes. Bugger me if I know why.
-  answer = answer >> 8 | (answer << 8 & 0xff00)
+    # Swap bytes. Bugger me if I know why.
+    answer = answer >> 8 | (answer << 8 & 0xff00)
 
-  return answer
+    return answer
 
 def receiveOnePing(mySocket, ID, timeout ):
-  timeLeft = timeout
+    timeLeft = timeout
 
-  while 1:
-    startedSelect = time.time()
-    whatReady = select.select( [mySocket], [], [], timeLeft )
-    howLongInSelect = ( time.time() - startedSelect )
+    while 1:
+        startedSelect = time.time()
+        whatReady = select.select( [mySocket], [], [], timeLeft )
+        howLongInSelect = ( time.time() - startedSelect )
 
-    if whatReady[0] == []: # Timeout
-      return -1
+        if whatReady[0] == []: # Timeout
+            return -1
 
-    timeReceived = time.time()
-    recPacket, addr = mySocket.recvfrom(1024)
-    icmpHeader = recPacket[20:28]
-    typ, code, checksum, packetID, sequence = struct.unpack( "bbHHh", icmpHeader )
+        timeReceived = time.time()
+        recPacket, addr = mySocket.recvfrom(1024)
+        icmpHeader = recPacket[20:28]
+        typ, code, checksum, packetID, sequence = struct.unpack( "bbHHh", icmpHeader )
 
-    if packetID == ID:
-      bytesInDouble = struct.calcsize( "d" )
-      timeSent = struct.unpack( "d", recPacket[ 28:28 + bytesInDouble ] )[0]
-      return timeReceived - timeSent
+        if packetID == ID:
+            bytesInDouble = struct.calcsize( "d" )
+            timeSent = struct.unpack( "d", recPacket[ 28:28 + bytesInDouble ] )[0]
+            return timeReceived - timeSent
 
-    timeLeft = timeLeft - howLongInSelect
+        timeLeft = timeLeft - howLongInSelect
 
-    if timeLeft <= 0:
-      return -1
+        if timeLeft <= 0:
+            return -1
 
 def sendOnePing( mySocket, destAddr, ID ):
-  # Header is type (8), code (8), checksum (16), id (16), sequence (16)
-  myChecksum = 0
-  
-  # Make a dummy heder with a 0 checksum.
-  header = struct.pack( "bbHHh", ICMP_ECHO_REQUEST, 0, myChecksum, ID, 1 )
-  bytesInDouble = struct.calcsize( "d" )
-  data = ( 192 - bytesInDouble ) * "Q"
-  data = struct.pack( "d", time.time() ) + data
-  
-  # Calculate the checksum on the data and the dummy header.
-  myChecksum = checksum( header + data )
-  
-  # Now that we have the right checksum, we put that in. It's just easier
-  # to make up a new header than to stuff it into the dummy.
-  header = struct.pack( "bbHHh", ICMP_ECHO_REQUEST, 0, 
-                        socket.htons( myChecksum ), ID, 1 )
-                        
-  packet = header + data
-  mySocket.sendto( packet, ( destAddr, 1 ) ) # Don't know about the 1 
+    # Header is type (8), code (8), checksum (16), id (16), sequence (16)
+    myChecksum = 0
+
+    # Make a dummy heder with a 0 checksum.
+    header = struct.pack( "bbHHh", ICMP_ECHO_REQUEST, 0, myChecksum, ID, 1 )
+    bytesInDouble = struct.calcsize( "d" )
+    data = ( 192 - bytesInDouble ) * "Q"
+    data = struct.pack( "d", time.time() ) + data
+
+    # Calculate the checksum on the data and the dummy header.
+    myChecksum = checksum( header + data )
+
+    # Now that we have the right checksum, we put that in. It's just easier
+    # to make up a new header than to stuff it into the dummy.
+    if prop.platform == 'darwin':
+        myChecksum = socket.htons( myChecksum ) & 0xffff
+    else:
+        myChecksum = socket.htons( myChecksum )
+
+    header = struct.pack( "bbHHh", ICMP_ECHO_REQUEST, 0,
+                        myChecksum, ID, 1 )
+
+    packet = header + data
+    mySocket.sendto( packet, ( destAddr, 1 ) ) # Don't know about the 1
 
 def doOne( destAddr, timeout=10 ):
-  # Returns either the delay (in seconds) or none on timeout.
-  icmp = socket.getprotobyname( "icmp" )
-  mySocket = socket.socket( socket.AF_INET,socket.SOCK_RAW,icmp )
-  myID = os.getpid() & 0xFFFF
-  sendOnePing( mySocket, destAddr, myID )
-  delay = receiveOnePing( mySocket, myID, timeout )
-  mySocket.close()
-  
-  return delay
+    # Returns either the delay (in seconds) or none on timeout.
+    icmp = socket.getprotobyname( "icmp" )
+    mySocket = socket.socket( socket.AF_INET,socket.SOCK_RAW,icmp )
+    myID = os.getpid() & 0xFFFF
+    sendOnePing( mySocket, destAddr, myID )
+    delay = receiveOnePing( mySocket, myID, timeout )
+    mySocket.close()
+
+    return delay
 
 
 def ping( host, timeout=1 ):
-  dest = socket.gethostbyname( host )
-  delay = doOne( dest, timeout )
-  return delay
+    dest = socket.gethostbyname( host )
+    delay = doOne( dest, timeout )
+    return delay
 
-# log_exception() adapted from ASPN:
-#    Recipe 65332 by Dirk Holtwick
-#    Recipe 52215 by Bryn Keller
-#    Adapted by Don Welch, 2004
 
-# TODO: Move this to logger module
-def log_exception():
-    """
-    Print the usual traceback information, followed by a listing of all the
-    local variables in each frame.
-    """
-    typ, value, tb = sys.exc_info()
+def checkPyQtImport():
+    # PyQt
+    try:
+        import qt
+    except ImportError:
+        log.error( "PyQt not installed. GUI not available. Exiting." )
+        return False
 
-    while 1:
-        if not tb.tb_next:
-            break
-        tb = tb.tb_next
+    # check version of Qt
+    qtMajor = int( qt.qVersion().split('.')[0] )
 
-    stack = []
-    f = tb.tb_frame
+    if qtMajor < MINIMUM_QT_MAJOR_VER:
 
-    while f:
-        stack.append(f)
-        f = f.f_back
+        log.error( "Incorrect version of Qt installed. Ver. 3.0.0 or greater required.")
+        return False
 
-    stack.reverse()
+    #check version of PyQt
+    try:
+        pyqtVersion = qt.PYQT_VERSION_STR
+    except:
+        pyqtVersion = qt.PYQT_VERSION
 
-    body = "Traceback (innermost last):\n"
-    lst = traceback.format_tb(tb) + traceback.format_exception_only(typ, value)
-    body = body + "%-20s %s" % (''.join( lst[:-1] ), lst[-1], )
+    while pyqtVersion.count('.') < 2:
+        pyqtVersion += '.0'
 
-    log.error( body )
+    (maj_ver, min_ver, pat_ver) = pyqtVersion.split('.')
+
+    if pyqtVersion.find( 'snapshot' ) >= 0:
+        log.warning( "A non-stable snapshot version of PyQt is installed.")
+    else:
+        try:
+            maj_ver = int(maj_ver)
+            min_ver = int(min_ver)
+            pat_ver = int(pat_ver)
+        except ValueError:
+            maj_ver, min_ver, pat_ver = 0, 0, 0
+
+        if maj_ver < MINIMUM_PYQT_MAJOR_VER or \
+            (maj_ver == MINIMUM_PYQT_MAJOR_VER and min_ver < MINIMUM_PYQT_MINOR_VER):
+            log.error( "This program may not function properly with the version of PyQt that is installed (%d.%d.%d)." % (maj_ver, min_ver, pat_ver) )
+            log.error( "Incorrect version of pyQt installed. Ver. %d.%d or greater required." % ( MINIMUM_PYQT_MAJOR_VER, MINIMUM_PYQT_MINOR_VER ) )
+            return False
+
+    return True
+
+
+def loadTranslators( app, user_config ):
+    #from qt import *
+    import qt
+    loc = None
+
+    if os.path.exists( user_config ):
+        # user_config contains executables we will run, so we
+        # must make sure it is a safe file, and refuse to run
+        # otherwise.
+        if not path_exists_safely( user_config ):
+            log.warning( "File %s has insecure permissions! File ignored." % user_config )
+        else:
+            config = ConfigParser.ConfigParser()
+            config.read( user_config )
+
+            if config.has_section( "ui" ):
+                loc = config.get( "ui", "loc" )
+
+                if not loc:
+                    loc = None
+
+    if loc is not None:
+
+        if loc.lower() == 'system':
+            loc = str(qt.QTextCodec.locale())
+
+        if loc.lower() != 'c':
+
+            log.debug( "Trying to load .qm file for %s locale." % loc )
+
+            dirs = [ prop.home_dir, prop.data_dir, prop.i18n_dir ]
+
+            trans = qt.QTranslator(None)
+
+            for dir in dirs:
+                qm_file = 'hplip_%s' % loc
+                loaded = trans.load( qm_file, dir)
+
+                if loaded:
+                    app.installTranslator( trans )
+                    break
+        else:
+            loc = None
+
+    if loc is None:
+        log.debug( "Using default 'C' locale" )
+    else:
+        log.debug( "Using locale: %s" % loc )
+
+    return loc
+
+try:
+    from string import Template # will fail in Python <= 2.3
+except ImportError:
+    # Code from Python 2.4 string.py
+    import re as _re
+
+    class _multimap:
+        """Helper class for combining multiple mappings.
+
+        Used by .{safe_,}substitute() to combine the mapping and keyword
+        arguments.
+        """
+        def __init__(self, primary, secondary):
+            self._primary = primary
+            self._secondary = secondary
+
+        def __getitem__(self, key):
+            try:
+                return self._primary[key]
+            except KeyError:
+                return self._secondary[key]
+
+
+    class _TemplateMetaclass(type):
+        pattern = r"""
+        %(delim)s(?:
+          (?P<escaped>%(delim)s) |   # Escape sequence of two delimiters
+          (?P<named>%(id)s)      |   # delimiter and a Python identifier
+          {(?P<braced>%(id)s)}   |   # delimiter and a braced identifier
+          (?P<invalid>)              # Other ill-formed delimiter exprs
+        )
+        """
+
+        def __init__(cls, name, bases, dct):
+            super(_TemplateMetaclass, cls).__init__(name, bases, dct)
+            if 'pattern' in dct:
+                pattern = cls.pattern
+            else:
+                pattern = _TemplateMetaclass.pattern % {
+                    'delim' : _re.escape(cls.delimiter),
+                    'id'    : cls.idpattern,
+                    }
+            cls.pattern = _re.compile(pattern, _re.IGNORECASE | _re.VERBOSE)
+
+
+    class Template:
+        """A string class for supporting $-substitutions."""
+        __metaclass__ = _TemplateMetaclass
+
+        delimiter = '$'
+        idpattern = r'[_a-z][_a-z0-9]*'
+
+        def __init__(self, template):
+            self.template = template
+
+        # Search for $$, $identifier, ${identifier}, and any bare $'s
+
+        def _invalid(self, mo):
+            i = mo.start('invalid')
+            lines = self.template[:i].splitlines(True)
+            if not lines:
+                colno = 1
+                lineno = 1
+            else:
+                colno = i - len(''.join(lines[:-1]))
+                lineno = len(lines)
+            raise ValueError('Invalid placeholder in string: line %d, col %d' %
+                             (lineno, colno))
+
+        def substitute(self, *args, **kws):
+            if len(args) > 1:
+                raise TypeError('Too many positional arguments')
+            if not args:
+                mapping = kws
+            elif kws:
+                mapping = _multimap(kws, args[0])
+            else:
+                mapping = args[0]
+            # Helper function for .sub()
+            def convert(mo):
+                # Check the most common path first.
+                named = mo.group('named') or mo.group('braced')
+                if named is not None:
+                    val = mapping[named]
+                    # We use this idiom instead of str() because the latter will
+                    # fail if val is a Unicode containing non-ASCII characters.
+                    return '%s' % val
+                if mo.group('escaped') is not None:
+                    return self.delimiter
+                if mo.group('invalid') is not None:
+                    self._invalid(mo)
+                raise ValueError('Unrecognized named group in pattern',
+                                 self.pattern)
+            return self.pattern.sub(convert, self.template)
+
+        def safe_substitute(self, *args, **kws):
+            if len(args) > 1:
+                raise TypeError('Too many positional arguments')
+            if not args:
+                mapping = kws
+            elif kws:
+                mapping = _multimap(kws, args[0])
+            else:
+                mapping = args[0]
+            # Helper function for .sub()
+            def convert(mo):
+                named = mo.group('named')
+                if named is not None:
+                    try:
+                        # We use this idiom instead of str() because the latter
+                        # will fail if val is a Unicode containing non-ASCII
+                        return '%s' % mapping[named]
+                    except KeyError:
+                        return self.delimiter + named
+                braced = mo.group('braced')
+                if braced is not None:
+                    try:
+                        return '%s' % mapping[braced]
+                    except KeyError:
+                        return self.delimiter + '{' + braced + '}'
+                if mo.group('escaped') is not None:
+                    return self.delimiter
+                if mo.group('invalid') is not None:
+                    return self.delimiter
+                raise ValueError('Unrecognized named group in pattern',
+                                 self.pattern)
+            return self.pattern.sub(convert, self.template)
 
 
 
+cat = lambda _ : Template(_).substitute(sys._getframe(1).f_globals, **sys._getframe(1).f_locals)
+
+
+class ModelParser:
+
+    def __init__( self ):
+        self.model = None
+        self.cur_model = None
+        self.stack = []
+        self.in_model = False
+        self.models = {}
+
+    def startElement( self, name, attrs ):
+        if name == 'models':
+            return
+
+        elif name == 'model':
+            self.model = {}
+            self.cur_model = str( attrs[ 'name' ] ).replace('_', ' ').\
+                replace( 'HP', '' ).replace( 'hp', '' ).strip()
+            self.in_model = True
+            self.stack = []
+
+        else:
+            self.stack.append( str(name).lower() )
+            if len(attrs):
+                for a in attrs:
+                    self.stack.append( str(a).lower() )
+                    try:
+                        i = int( attrs[a] )
+                    except ValueError:
+                        i = str( attrs[a] )
+
+                    self.model[ str( '-'.join( self.stack ) ) ] = i
+                    self.stack.pop()
+
+
+    def endElement( self, name ):
+        if name == 'model':
+            self.in_model = False
+
+            if self.cur_model in self.models:
+                log.error( "Duplicate model in XML: %s" % self.cur_model )
+                raise Error( ERROR_INTERNAL )
+
+            self.models[ self.cur_model ] = self.model
+
+            self.model = None
+        elif name == 'models':
+            return
+        else:
+            self.stack.pop()
+
+
+    def charData( self, data ):
+        data = str(data).strip()
+        if data and self.model is not None and self.stack:
+            self.model[ str( '-'.join( self.stack ) ) ] = str( data )
+
+    def loadModels( self, filename, untested=False ):
+        parser = xml.parsers.expat.ParserCreate()
+        parser.StartElementHandler = self.startElement
+        parser.EndElementHandler = self.endElement
+        parser.CharacterDataHandler = self.charData
+        try:
+            parser.Parse( open( filename ).read(), True )
+        except xml.parsers.expat.ExpatError, e:
+            log.error( "XML file parse error: %s" % e )
+            raise Error( ERROR_INTERNAL )
+
+        return self.models
+
+
+identity = string.maketrans('','')
+unprintable = identity.translate(identity, string.printable)
+
+def printable(s):
+    return s.translate(identity, unprintable)
+
+
+def usage_logging(formatter, space_after=False):
+    log.info( formatter.compose( ( "Set the logging level:", "-l<level> or --logging=<level>" ) ) )
+    log.info( formatter.compose( ( "", "<level>: none, info*, error, warn, debug (*default)" ), space_after ) )
+
+def usage_help(formatter, space_after=False):
+    log.info( formatter.compose( ( "This help information:", "-h or --help" ), space_after ) )
+
+def usage_device(formatter, space_after=False):
+    log.info( formatter.compose( ( "To specify a device-URI:", "-d<device-uri> or --device=<device-uri>" ) ) )
+
+def usage_printer(formatter, space_after=False):
+    log.info( formatter.compose( ( "To specify a CUPS printer:", "-p<printer> or --printer=<printer>" ), space_after ) )
+
+def usage_options():
+    log.info(bold("[OPTIONS]"))
+
+def usage_bus(formatter, space_after=False):
+    log.info( formatter.compose( ( "Bus to probe (if device not specified):", "-b<bus> or --bus=<bus>" ) ) )
+    log.info( formatter.compose( ( "",
+        "<bus>: cups*, usb*, net, bt, fw, par* (*default) (Note: bw and bt not supported)" ), space_after ) )
+
+def usage_examples():
+    log.info(bold("Examples:"))
+
+def usage_notes():
+    log.info(bold("**NOTES"))
+    log.info( """\t1. If device or printer is not specified, the local device bus\n""" \
+              """ \t   is probed and the program enters interactive mode.\n""" \
+              """\t2. If -p* is specified, the default CUPS printer will be used.\n""" )
+
+def ttysize():
+    ln1 = commands.getoutput( 'stty -a' ).splitlines()[0]
+    vals = {'rows':None, 'columns':None}
+    for ph in ln1.split(';'):
+        x = ph.split()
+        if len(x) == 2:
+            vals[x[0]] = x[1]
+            vals[x[1]] = x[0]
+    return int( vals['rows'] ), int( vals['columns'] )
+
+
+def usage_formatter(override=0):
+    rows, cols = ttysize()
+    
+    if override:
+        col1 = override
+        col2 = cols - col1 - 8
+    else:
+        col1 = int(cols / 3) - 8
+        col2 = cols - col1 - 8
+    
+    return TextFormatter( ( {'width': col1, 'margin' : 2},
+                            {'width': col2, 'margin' : 2}, ))
+
+
+def any(S,f=lambda x:x):
+    for x in S:
+        if f(x): return True
+    return False
+
+def all(S,f=lambda x:x):
+    for x in S:
+        if not f(x): return False
+    return True
+
+
