@@ -63,6 +63,82 @@ Device::~Device()
    pthread_mutex_destroy(&mutex);
 }
 
+//Device::Write
+//!  Kernel write with timeout. File descriptor must be opened with O_NONBLOCK.
+/*!
+******************************************************************************/
+int Device::Write(int fd, const void *buf, int size)
+{
+   struct timeval tmo;
+   fd_set master;
+   fd_set writefd;
+   int maxfd, ret, len=0;
+
+   FD_ZERO(&master);
+   FD_SET(fd, &master);
+   maxfd = fd;
+   tmo.tv_sec = EXCEPTION_TIMEOUT;
+   tmo.tv_usec = 0;
+
+   while(1)
+   {
+      writefd = master;
+      if ((ret = select(maxfd+1, NULL, &writefd, NULL, &tmo)) == 0)
+      {
+         len = -1;
+         break;   /* timeout */
+      }
+      len = write(fd, buf, size);
+      if (len < 0)
+         if (errno == EAGAIN)
+            continue;
+      break;
+   }
+
+   return len;
+}
+
+//Device::Read
+//!  Kernel read with timeout. File descriptor must be opened with O_NONBLOCK.
+/*!
+******************************************************************************/
+int Device::Read(int fd, void *buf, int size, int sec, int usec)
+{
+   struct timeval tmo;
+   fd_set master;
+   fd_set readfd;
+   int maxfd, ret, len=0;
+
+   FD_ZERO(&master);
+   FD_SET(fd, &master);
+   maxfd = fd;
+   tmo.tv_sec = sec;
+   tmo.tv_usec = usec;
+
+   while(1)
+   {
+      readfd = master;
+      if ((ret = select(maxfd+1, &readfd, NULL, NULL, &tmo)) == 0)
+      {
+         len = -1;
+         break;   /* timeout */
+      }
+      len = read(fd, buf, size);
+      if (len == 0)
+      {
+         // Zero reads wasts cpu resources, but this appears to be normal. 
+         //         syslog(LOG_ERR, "zero read\n");   
+         continue;
+      }
+      if (len < 0)
+         if (errno == EAGAIN)
+            continue;
+      break;
+   }
+
+   return len;
+}
+
 int Device::DeviceID(char *buffer, int size)
 {
    int len=0, maxSize;
@@ -73,7 +149,7 @@ int Device::DeviceID(char *buffer, int size)
    {
      //      if (errno == ENODEV)
      //         len = -1;
-      syslog(LOG_ERR, "unable to read Device::DeviceID: %m\n");
+      syslog(LOG_ERR, "unable to read uri:%s Device::DeviceID: %m\n", URI);
       goto bugout;
    }
 
@@ -161,7 +237,7 @@ int Device::PowerUp()
    else
       return 0;  /* must be laserjet, don't do power-up */
 
-   write(OpenFD, Venice_Power_On, sizeof(Venice_Power_On));  
+   Write(OpenFD, Venice_Power_On, sizeof(Venice_Power_On));  
    sleep(2);
 
    return 0;
@@ -173,7 +249,6 @@ int Device::Open(char *sendBuf, int *result)
    char uriModel[128];
    char model[128];
    int len=0;
-   unsigned char nullByte=0;
 
    *result = R_AOK;
 
@@ -190,6 +265,8 @@ int Device::Open(char *sendBuf, int *result)
       if ((OpenFD = open(dev, O_RDWR | O_NONBLOCK | O_EXCL)) < 0) 
       {
          *result = R_IO_ERROR;
+         if (strcmp(uriModel, "ANY") != 0)
+            syslog(LOG_ERR, "unable to Device::Open %s: %m\n", URI);
          goto blackout;
       }
 
@@ -203,15 +280,6 @@ int Device::Open(char *sendBuf, int *result)
 
       if (pSys->IsHP(ID) && strcmp(uriModel, "ANY") != 0)
       {
-#if 0
-         /* Don't need MLC reset if crossbow. */
-         if (strstr(ID, "CMD:LDL") == NULL)
-         {
-            ioctl(OpenFD, LPIOC_HP_SET_CHANNEL, 78);  /* reset MLC (ECP channel-78) */
-            pSys->Write(OpenFD, &nullByte, 1);  
-            ioctl(OpenFD, LPIOC_HP_SET_CHANNEL, 0);   /* set raw mode (ECP channel-0) */
-         }
-#endif
          PowerUp();
       }
    }
@@ -277,14 +345,13 @@ int Device::GetDeviceID(char *sendBuf, int slen, int *result)
 {
    char res[] = "msg=DeviceIDResult\nresult-code=%d\n";
    int len=0, idLen;
-   char id[1024];
 
    *result = R_AOK;
 
 //   if (pthread_mutex_trylock(&mutex) == 0)
    if (pthread_mutex_lock(&mutex) == 0)
    {
-      idLen = DeviceID(id, sizeof(id));  /* get new copy */
+      idLen = DeviceID(ID, sizeof(ID));  /* get new copy */
       pthread_mutex_unlock(&mutex);
 
       if (idLen == 0)
@@ -314,7 +381,7 @@ int Device::GetDeviceID(char *sendBuf, int slen, int *result)
       goto hijmp;
    }
    len = sprintf(sendBuf, "msg=DeviceIDResult\nresult-code=%d\nlength=%d\ndata:\n", *result, idLen); 
-   memcpy(&sendBuf[len], id, idLen);
+   memcpy(&sendBuf[len], ID, idLen);
    len += idLen; 
 //   sendBuf[len] = '\n';  /* for ascii data */
 
@@ -458,10 +525,9 @@ rjmp:
 //!  Create channel object given the service name.
 /*!
 ******************************************************************************/
-Channel *Device::NewChannel(unsigned char sockid, char *io_mode, char *flow_ctl)
+Channel *Device::NewChannel(unsigned char sockid)
 {
    Channel *pC=NULL;
-   MsgAttributes ma;
    int i, n, mode;
 
    /* Check for existing name service already open. */
@@ -485,14 +551,11 @@ Channel *Device::NewChannel(unsigned char sockid, char *io_mode, char *flow_ctl)
    if (ChannelCnt >= MAX_CHANNEL)
       goto bugout;
 
-   /* For printing, check the IO mode (RAW or MLC). All other services use MLC. */
+   /* Get requested IO mode. */
    if (sockid == PRINT_CHANNEL)
-   {
-      /* Get requested IO mode. */
-      mode = (strcasecmp(io_mode, "mlc") == 0) ? MLC_MODE : RAW_MODE;
-   }
+      mode = GetPrintMode();
    else
-      mode = MLC_MODE;
+      mode = GetMfpMode();
 
    /* Make sure requested io mode matches any current io mode. */
    if (ChannelCnt && ChannelMode != mode)
@@ -506,10 +569,7 @@ Channel *Device::NewChannel(unsigned char sockid, char *io_mode, char *flow_ctl)
          if (mode == RAW_MODE)
             pC = new RawChannel(this);  /* constructor sets ClientCnt=1 */
          else
-         {
             pC = new MlcChannel(this);  /* constructor sets ClientCnt=1 */
-            ((MlcChannel *)pC)->SetMiser((strcasecmp(flow_ctl, "miser") == 0) ? 1 : 0);
-         }
 
          pC->SetIndex(i);
          pC->SetSocketID(sockid);
@@ -542,7 +602,7 @@ int Device::DelChannel(int chan)
    return 0;
 }
 
-int Device::ChannelOpen(char *sn, char *io_mode, char *flow_ctl, int *channel, char *sendBuf, int *result)
+int Device::ChannelOpen(char *sn, int *channel, char *sendBuf, int *result)
 {
    char res[] = "msg=ChannelOpenResult\nresult-code=%d\n";
    Channel *pC;
@@ -575,7 +635,7 @@ int Device::ChannelOpen(char *sn, char *io_mode, char *flow_ctl, int *channel, c
    }
    else
    {
-      syslog(LOG_ERR, "unsupported service Device::ChannelOpen: %s\n", sn);
+      syslog(LOG_ERR, "unsupported service uri:%s Device::ChannelOpen: %s\n", URI, sn);
       len = sprintf(sendBuf, res, R_INVALID_SN);
       goto bugout;
    }
@@ -583,9 +643,9 @@ int Device::ChannelOpen(char *sn, char *io_mode, char *flow_ctl, int *channel, c
    //   if (pthread_mutex_trylock(&mutex) == 0)
    if (pthread_mutex_lock(&mutex) == 0)
    {
-      if ((pC = NewChannel(sockid, io_mode, flow_ctl)) == NULL)
+      if ((pC = NewChannel(sockid)) == NULL)
       {
-         syslog(LOG_ERR, "service busy Device::ChannelOpen: %s\n", sn);
+         syslog(LOG_ERR, "service busy uri:%s Device::ChannelOpen: %s\n", URI, sn);
          *result = R_CHANNEL_BUSY;
          len = sprintf(sendBuf, res, *result);
       }
@@ -605,7 +665,7 @@ int Device::ChannelOpen(char *sn, char *io_mode, char *flow_ctl, int *channel, c
    }
    else
    {
-      syslog(LOG_ERR, "unable to lock Device::ChannelOpen: %m\n");
+      syslog(LOG_ERR, "unable to lock uri:%s Device::ChannelOpen: %m\n", URI);
       *result = R_IO_ERROR;
       len = sprintf(sendBuf, res, *result);  
       goto bugout;
@@ -623,7 +683,7 @@ int Device::ChannelClose(int channel, char *sendBuf, int *result)
 
    if (pC == NULL)
    {
-      syslog(LOG_ERR, "invalid channel descriptor Device::ChannelClose: %d\n", channel);
+      syslog(LOG_ERR, "invalid channel descriptor uri:%s Device::ChannelClose: %d\n", URI, channel);
       *result = R_INVALID_CHANNEL_ID;
       len = sprintf(sendBuf, res, *result);  
       goto bugout;
@@ -646,7 +706,7 @@ int Device::ChannelClose(int channel, char *sendBuf, int *result)
    }
    else
    {
-      syslog(LOG_ERR, "unable to lock Device::ChannelClose: %m\n");
+      syslog(LOG_ERR, "unable to lock uri:%s Device::ChannelClose: %m\n", URI);
       *result = R_IO_ERROR;
       len = sprintf(sendBuf, res, *result);  
       goto bugout;
