@@ -70,7 +70,8 @@ System::System()
 
    bzero(&sin, sizeof(sin));
    sin.sin_family = AF_INET;
-   sin.sin_addr.s_addr = INADDR_ANY;
+//   sin.sin_addr.s_addr = INADDR_ANY;
+   sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
    sin.sin_port = htons(GetHpiodPortNumber());
 
    if (bind(Permsd, (struct sockaddr *)&sin, sizeof(sin)) == -1) 
@@ -150,98 +151,6 @@ int System::ReadConfig()
    return 0;
 }
 
-//System::Write
-//!  Kernel write with timeout. File descriptor must be opened with O_NONBLOCK.
-/*!
-******************************************************************************/
-int System::Write(int fd, const void *buf, int size)
-{
-   struct timeval tmo;
-   fd_set master;
-   fd_set writefd;
-   int maxfd, ret, len=0;
-
-   if (fd<0)
-   {
-      syslog(LOG_ERR, "invalid System::Write: %d\n", fd);  /* extra sanity check */
-      len = -2;
-      goto bugout;
-   }
-
-   FD_ZERO(&master);
-   FD_SET(fd, &master);
-   maxfd = fd;
-   tmo.tv_sec = EXCEPTION_TIMEOUT;
-   tmo.tv_usec = 0;
-
-   while(1)
-   {
-      writefd = master;
-      if ((ret = select(maxfd+1, NULL, &writefd, NULL, &tmo)) == 0)
-      {
-         len = -1;
-         break;   /* timeout */
-      }
-      len = write(fd, buf, size);
-      if (len < 0)
-         if (errno == EAGAIN)
-            continue;
-      break;
-   }
-
-bugout:
-   return len;
-}
-
-//System::Read
-//!  Kernel read with timeout. File descriptor must be opened with O_NONBLOCK.
-/*!
-******************************************************************************/
-int System::Read(int fd, void *buf, int size, int sec, int usec)
-{
-   struct timeval tmo;
-   fd_set master;
-   fd_set readfd;
-   int maxfd, ret, len=0;
-
-   if (fd<0)
-   {
-      syslog(LOG_ERR, "invalid System::Read: %d\n", fd);  /* extra sanity check */
-      len = -2;
-      goto bugout;
-   }
-
-   FD_ZERO(&master);
-   FD_SET(fd, &master);
-   maxfd = fd;
-   tmo.tv_sec = sec;
-   tmo.tv_usec = usec;
-
-   while(1)
-   {
-      readfd = master;
-      if ((ret = select(maxfd+1, &readfd, NULL, NULL, &tmo)) == 0)
-      {
-         len = -1;
-         break;   /* timeout */
-      }
-      len = read(fd, buf, size);
-      if (len == 0)
-      {
-         // Zero reads wasts cpu resources, but this appears to be normal. 
-         //         syslog(LOG_ERR, "zero read\n");   
-         continue;
-      }
-      if (len < 0)
-         if (errno == EAGAIN)
-            continue;
-      break;
-   }
-
-bugout:
-   return len;
-}
-
 //System::GetPair
 //!  Get key value pair from buffer. Assumes one key value pair per line.
 //!  Note, end-of-line '/n' is stripped out. Both key/value pairs are zero
@@ -315,7 +224,7 @@ int System::IsHP(char *id)
 }
 
 //System::IsUdev
-//! Given a usb device node, determine if this is a udev node (a non-legacy device node).
+//! Given a usb or parallel device node, determine if this is a udev node (a non-legacy device node).
 /*!
 ******************************************************************************/
 int System::IsUdev(char *dnode)
@@ -327,11 +236,18 @@ int System::IsUdev(char *dnode)
       return 1;
 
    p+=11;
-
    n = strtol(p, &tail, 10);
    if (n < 0 || n > 15)
      return 1;
 
+   if ((p = strstr(dnode, "/dev/parport")) == NULL)
+      return 1;
+
+   p+=12;
+   n = strtol(p, &tail, 10);
+   if (n < 0 || n > 3)
+     return 1;
+
    return 0; /* looks like a legacy device node */
 }
 
@@ -342,7 +258,7 @@ int System::IsUdev(char *dnode)
 int System::GetModel(char *id, char *buf, int bufSize)
 {
    char *pMd;
-   int i;
+   int i, j, dd=0;
 
    buf[0] = 0;
 
@@ -353,16 +269,31 @@ int System::GetModel(char *id, char *buf, int bufSize)
    else
       return 0;
 
-   for (i=0; (pMd[i] != ';') && (i < bufSize); i++)
+   for (i=0; pMd[i] == ' ' && i < bufSize; i++);  /* eat leading white space */
+
+   for (j=0; (pMd[i] != ';') && (i < bufSize); i++)
    {
       if (pMd[i]==' ' || pMd[i]=='/')
-         buf[i] = '_';   /* convert space to "_" */
+      {
+         /* Remove double spaces. */
+         if (!dd)
+         { 
+            buf[j++] = '_';   /* convert space to "_" */
+            dd=1;              
+         }
+      }
       else
-         buf[i] = pMd[i];
+      {
+         buf[j++] = pMd[i];
+         dd=0;       
+      }
    }
-   buf[i] = 0;
 
-   return i;
+   for (j--; buf[j] == '_' && j > 0; j--);  /* eat trailing white space */
+
+   buf[++j] = 0;
+
+   return j;    /* return size does not include zero terminator */
 }
 
 //System::GetSerialNum
@@ -401,7 +332,14 @@ int System::GetURIDataLink(char *uri, char *buf, int bufSize)
 
    buf[0] = 0;
 
-   if (strcasestr(uri, "usb/") != NULL)
+   if (strcasestr(uri, "hp:/usb") != NULL)
+   {
+      if ((p = strcasestr(uri, "device=")) != NULL)
+         p+=7;
+      else
+         return 0;
+   }
+   else if (strcasestr(uri, "hp:/par") != NULL)
    {
       if ((p = strcasestr(uri, "device=")) != NULL)
          p+=7;
@@ -497,7 +435,7 @@ int System::GetURISerial(char *uri, char *buf, int bufSize)
 ******************************************************************************/
 int System::GeneralizeURI(MsgAttributes *ma)
 {
-   char dev[255];
+   MsgAttributes ma2;
    char model[128];
    char serial[128];
    char uriModel[128];
@@ -507,7 +445,7 @@ int System::GeneralizeURI(MsgAttributes *ma)
    int i, len, found=0, result;
    Device *pD;
 
-   if (strcasestr(ma->uri, "usb/") == NULL && strcasestr(ma->uri, "net/") == NULL)
+   if (strcasestr(ma->uri, "hp:/usb") == NULL && strcasestr(ma->uri, "hp:/net") == NULL && strcasestr(ma->uri, "hp:/par") == NULL)
       return 1;  /* invalid uri */
 
    if (strcasestr(ma->uri, "device=") != NULL)
@@ -524,9 +462,10 @@ int System::GeneralizeURI(MsgAttributes *ma)
       
    for (i = 0; i < 16 && !found; i ++)
    {
-      sprintf(dev, "hp:/usb/ANY?device=/dev/usb/lp%d", i);
+      ma2.prt_mode = RAW_MODE;
+      sprintf(ma2.uri, "hp:/usb/ANY?device=/dev/usb/lp%d", i);
 
-      pD = NewDevice(dev);
+      pD = NewDevice(&ma2);
       len = pD->Open(sendBuf, &result);
       if (result == R_AOK)
       {
@@ -560,6 +499,7 @@ int System::GeneralizeURI(MsgAttributes *ma)
 ******************************************************************************/
 int System::UsbDiscovery(char *lst, int *cnt)
 {
+   MsgAttributes ma2;
    char dev[255];
    char model[128];
    char serial[128];
@@ -568,11 +508,12 @@ int System::UsbDiscovery(char *lst, int *cnt)
    int i, len, size=0, result;
    Device *pD;
 
-   for (*cnt=0, i=0; i < 16; i++)
+   for (i=0; i < 16; i++)
    {
-      sprintf(dev, "hp:/usb/ANY?device=/dev/usb/lp%d", i);
+      ma2.prt_mode = RAW_MODE;
+      sprintf(ma2.uri, "hp:/usb/ANY?device=/dev/usb/lp%d", i);
 
-      pD = NewDevice(dev);
+      pD = NewDevice(&ma2);
       len = pD->Open(sendBuf, &result);
       if (result == R_AOK)
       {
@@ -598,17 +539,71 @@ int System::UsbDiscovery(char *lst, int *cnt)
    return size;
 }
 
+//System::ParDiscovery
+//! Walk the parallel ports looking for HP products. 
+/*!
+******************************************************************************/
+int System::ParDiscovery(char *lst, int *cnt)
+{
+   MsgAttributes ma2;
+   char dev[255];
+   char model[128];
+   char sendBuf[2048];
+   char *id;
+   int i, len, size=0, result;
+   Device *pD;
+
+   for (i=0; i < 4; i++)
+   {
+      ma2.prt_mode = RAW_MODE;
+      sprintf(ma2.uri, "hp:/par/ANY?device=/dev/parport%d", i);
+
+      pD = NewDevice(&ma2);
+      len = pD->Open(sendBuf, &result);
+      if (result == R_AOK)
+      {
+         id = pD->GetID(); /* use cached copy */
+
+         if (id[0] != 0 && IsHP(id))
+         {
+            GetModel(id, model, sizeof(model));
+            sprintf(dev, "hp:/par/%s?device=/dev/parport%d", model, i);
+            size += sprintf(lst+size,"direct %s \"HP %s\" \"%s\"\n", dev, model, dev);
+            *cnt+=1;
+         }
+      }
+
+      pD->Close(sendBuf, &result);
+      DelDevice(pD->GetIndex());
+   }
+
+   return size;
+}
+
 //System::ProbeDevices
 //!  Perform hp device discovery. Works simultaneously with other open clients.
 /*!
 ******************************************************************************/
-int System::ProbeDevices(char *sendBuf)
+int System::ProbeDevices(char *sendBuf, char *bus)
 {
    char lst[LINE_SIZE*MAX_DEVICE];
-   int len, lstLen, cnt;
+   int len, lstLen, cnt=0;
 
    lst[0] = 0;
-   lstLen = UsbDiscovery(lst, &cnt);
+
+   if (strcasecmp(bus, "usb") == 0)
+   {
+      lstLen = UsbDiscovery(lst, &cnt);
+   }
+   else if (strcasecmp(bus, "par") == 0)
+   {
+      lstLen = ParDiscovery(lst, &cnt);
+   }
+   else
+   {
+      lstLen = UsbDiscovery(lst, &cnt);
+      lstLen += ParDiscovery(lst+lstLen, &cnt);
+   }
 
    len = sprintf(sendBuf, "msg=ProbeDevicesResult\nresult-code=%d\nnum-devices=%d\nlength=%d\ndata:\n%s", R_AOK, cnt, lstLen, lst); 
 
@@ -717,7 +712,8 @@ int System::SetSnmp(char *ip, int port, char *szoid, int type, unsigned char *bu
    struct snmp_pdu *response=NULL;
    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;
-   int i, len=0;
+   int len=0;
+   unsigned int i;
    uint32_t val;
 
    *result = R_IO_ERROR;
@@ -742,7 +738,7 @@ int System::SetSnmp(char *ip, int port, char *szoid, int type, unsigned char *bu
       case PML_DT_ENUMERATION:
       case PML_DT_SIGNED_INTEGER:
          /* Convert PML big-endian to SNMP little-endian byte stream. */
-         for(i=0, val=0; i<size && i<sizeof(val); i++)    
+         for(i=0, val=0; i<(unsigned int)size && i<sizeof(val); i++)    
             val = ((val << 8) | buffer[i]);
          snmp_pdu_add_variable(pdu, anOID, anOID_len, ASN_INTEGER, (unsigned char *)&val, sizeof(val));
          break;
@@ -821,18 +817,18 @@ int System::GetSnmp(char *ip, int port, char *szoid, unsigned char *buffer, int
             *type = PML_DT_SIGNED_INTEGER;
 
             /* Convert SNMP little-endian to PML big-endian byte stream. */
-            len = (sizeof(uint32_t) < size) ? sizeof(uint32_t) : size;
+            len = (sizeof(uint32_t) < (unsigned int)size) ? sizeof(uint32_t) : size;
             val = *vars->val.integer;
-            for(i=len-1; i>0; i--)
+            for(i=len; i>0; i--)
             {
-               tmp[i] = val & 0xff;
+               tmp[i-1] = val & 0xff;
                val >>= 8;
             }
 
             /* Remove any in-significant bytes. */
-            for (; tmp[i]==0 && i<len; i++)
+            for (; tmp[i-1]==0 && i<=len; i++)
                ;
-            len -= i;
+            len -= --i;
 
             memcpy(buffer, tmp+i, len);
             break;
@@ -841,7 +837,7 @@ int System::GetSnmp(char *ip, int port, char *szoid, unsigned char *buffer, int
             break;
          case ASN_OCTET_STR:
             *type = PML_DT_STRING;
-            len = (vars->val_len < size) ? vars->val_len : size;
+            len = (vars->val_len < (unsigned int)size) ? vars->val_len : size;
             memcpy(buffer, vars->val.string, len);
             break;
          default:
@@ -889,7 +885,7 @@ int System::SetPml(int device, int channel, char *snmp_oid, int type, unsigned c
    char message[BUFFER_SIZE];
    unsigned char oid[LINE_SIZE];
    unsigned char *p=(unsigned char *)message;
-   int len, dLen, result, reply, status, dt;
+   int len, dLen, result, reply, status;
    MsgAttributes ma;
    Device *pD=pDevice[device];
 
@@ -900,7 +896,17 @@ int System::SetPml(int device, int channel, char *snmp_oid, int type, unsigned c
       goto bugout;       
    }   
       
-   if (strcasestr(pD->GetURI(), "usb/") != NULL)
+   if (strcasestr(pD->GetURI(), "net/") != NULL)
+   {
+      /* Process pml via snmp. */
+      SetSnmp(((JetDirectDevice *)pD)->GetIP(), ((JetDirectDevice *)pD)->GetPort(), snmp_oid, type, data, dataLen, &status, &result);
+      if (result != R_AOK)
+      {
+         len = sprintf(sendBuf, res, R_IO_ERROR);
+         goto bugout;       
+      }
+   }       
+   else
    {
       /* Process pml via local transport. */
 
@@ -926,7 +932,7 @@ int System::SetPml(int device, int channel, char *snmp_oid, int type, unsigned c
       }    
 
       len = pD->ReadData(PML_MAX_DATALEN, channel, EXCEPTION_TIMEOUT, message, sizeof(message), &result);
-      if (result != R_AOK)
+      if (result != R_AOK || len == 0)
       {
          len = sprintf(sendBuf, res, R_IO_ERROR);
          goto bugout;       
@@ -946,16 +952,6 @@ int System::SetPml(int device, int channel, char *snmp_oid, int type, unsigned c
          goto bugout;       
       }   
    }
-   else
-   {
-      /* Process pml via snmp. */
-      SetSnmp(((JetDirectDevice *)pD)->GetIP(), ((JetDirectDevice *)pD)->GetPort(), snmp_oid, type, data, dataLen, &status, &result);
-      if (result != R_AOK)
-      {
-         len = sprintf(sendBuf, res, R_IO_ERROR);
-         goto bugout;       
-      }
-   }       
    
    len = sprintf(sendBuf, "msg=SetPMLResult\nresult-code=%d\npml-result-code=%d\n", R_AOK, status); 
 
@@ -978,7 +974,18 @@ int System::GetPml(int device, int channel, char *snmp_oid, char *sendBuf)
    MsgAttributes ma;
    Device *pD=pDevice[device];
 
-   if (strcasestr(pD->GetURI(), "usb/") != NULL)
+   if (strcasestr(pD->GetURI(), "net/") != NULL)
+   {
+      /* Process pml via snmp. */
+      dLen = GetSnmp(((JetDirectDevice *)pD)->GetIP(), ((JetDirectDevice *)pD)->GetPort(), snmp_oid, (unsigned char *)message, sizeof(message), &dt, &status, &result);
+      if (result != R_AOK)
+      {
+         len = sprintf(sendBuf, res, R_IO_ERROR);
+         goto bugout;       
+      }
+      p = (unsigned char *)message;    
+   }       
+   else
    {
       /* Process pml via local transport. */
 
@@ -997,7 +1004,7 @@ int System::GetPml(int device, int channel, char *snmp_oid, char *sendBuf)
       }    
 
       len = pD->ReadData(PML_MAX_DATALEN, channel, EXCEPTION_TIMEOUT, message, sizeof(message), &result);
-      if (result != R_AOK)
+      if (result != R_AOK || len == 0)
       {
          len = sprintf(sendBuf, res, R_IO_ERROR);
          goto bugout;       
@@ -1040,17 +1047,6 @@ int System::GetPml(int device, int channel, char *snmp_oid, char *sendBuf)
       dLen = ((*p & 0x3) << 8 | *(p+1));         /* read 10 bit len from 2 byte field */
       p += 2;                               /* eat type and length */
    }
-   else
-   {
-      /* Process pml via snmp. */
-      dLen = GetSnmp(((JetDirectDevice *)pD)->GetIP(), ((JetDirectDevice *)pD)->GetPort(), snmp_oid, (unsigned char *)message, sizeof(message), &dt, &status, &result);
-      if (result != R_AOK)
-      {
-         len = sprintf(sendBuf, res, R_IO_ERROR);
-         goto bugout;       
-      }
-      p = (unsigned char *)message;    
-   }       
    
    len = sprintf(sendBuf, "msg=GetPMLResult\nresult-code=%d\npml-result-code=%d\ntype=%d\nlength=%d\ndata:\n", R_AOK, status, dt, dLen); 
    memcpy(&sendBuf[len], p, dLen);
@@ -1067,7 +1063,7 @@ bugout:
 int System::MakeUriFromIP(char *ip, int port, char *sendBuf)
 {
    char res[] = "msg=MakeURIResult\nresult-code=%d\n";
-   int len=0, maxSize, result, dt, status;
+   int len=0, result, dt, status;
    char devid[1024];
    char model[128];
 
@@ -1098,36 +1094,37 @@ bugout:
    return len;
 }
 
-//System::MakeUriFromDevice
-//!  Given an device node read deviceID and create valid URI.
+//System::MakeUriFromUsb
+//!  Given a device node read deviceID and create valid URI.
 /*!
 ******************************************************************************/
-int System::MakeUriFromDevice(char *dnode, char *sendBuf)
+int System::MakeUriFromUsb(char *dnode, char *sendBuf)
 {
    char res[] = "msg=MakeURIResult\nresult-code=%d\n";
-   char dev[255];
+   MsgAttributes ma;
    char model[128];
    char serial[128];
    char dummyBuf[2048];
    char *id;
-   int len, size=0, result;
+   int len, result;
    Device *pD=NULL;
 
    len = sprintf(sendBuf, res, R_INVALID_DEVICE_NODE);
 
    if (dnode[0]==0)
    {
-      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromDevice: line %d\n", dnode, __LINE__);
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromUsb: %s %d\n", dnode, __FILE__, __LINE__);
       goto bugout;
    }
 
-   sprintf(dev, "hp:/usb/ANY?device=%s", dnode);  /* dnode = /dev/usb/lpxx or /dev/udev_name */
+   ma.prt_mode = RAW_MODE;
+   sprintf(ma.uri, "hp:/usb/ANY?device=%s", dnode);  /* dnode = /dev/usb/lpxx or /dev/udev_name */
 
-   pD = NewDevice(dev);
+   pD = NewDevice(&ma);
    pD->Open(dummyBuf, &result);
    if (result != R_AOK)
    {
-      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromDevice: line %d\n", dnode, __LINE__);
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromPP: %s %d\n", dnode, __FILE__, __LINE__);
       goto bugout;
    }
 
@@ -1135,7 +1132,7 @@ int System::MakeUriFromDevice(char *dnode, char *sendBuf)
 
    if (!IsHP(id))
    {
-      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromDevice: line %d\n", dnode, __LINE__);
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromPP: %s %d\n", dnode, __FILE__, __LINE__);
       goto bugout;
    }
 
@@ -1156,6 +1153,61 @@ bugout:
    return len;
 }
 
+//System::MakeUriFromPar
+//!  Given a device node read deviceID and create valid URI.
+/*!
+******************************************************************************/
+int System::MakeUriFromPar(char *dnode, char *sendBuf)
+{
+   char res[] = "msg=MakeURIResult\nresult-code=%d\n";
+   MsgAttributes ma;
+   char model[128];
+   char dummyBuf[2048];
+   char *id;
+   int len, result;
+   Device *pD=NULL;
+
+   len = sprintf(sendBuf, res, R_INVALID_DEVICE_NODE);
+
+   if (dnode[0]==0)
+   {
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromPar: %s %d\n", dnode, __FILE__, __LINE__);
+      goto bugout;
+   }
+
+   ma.prt_mode = RAW_MODE;
+
+   sprintf(ma.uri, "hp:/par/ANY?device=%s", dnode);  /* dnode = /dev/parportx or /dev/udev_name */
+
+   pD = NewDevice(&ma);
+   pD->Open(dummyBuf, &result);
+   if (result != R_AOK)
+   {
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromPar: %s %d\n", dnode, __FILE__, __LINE__);
+      goto bugout;
+   }
+
+   id = pD->GetID(); /* use cached copy */
+
+   if (!IsHP(id))
+   {
+      syslog(LOG_ERR, "invalid device node %s System::MakeUriFromPar: %s %d\n", dnode, __FILE__, __LINE__);
+      goto bugout;
+   }
+
+   GetModel(id, model, sizeof(model));
+   len = sprintf(sendBuf, "msg=MakeURIResult\nresult-code=%d\ndevice-uri=hp:/par/%s?device=%s\n", R_AOK, model, dnode); 
+
+bugout:
+   if (pD != NULL)
+   {
+      pD->Close(dummyBuf, &result);
+      DelDevice(pD->GetIndex());
+   }
+
+   return len;
+}
+
 //System::ParseMsg
 //!  Parse and convert all key value pairs in message. Do sanity check on values.
 /*!
@@ -1170,11 +1222,14 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
    ma->cmd[0] = 0;
    ma->uri[0] = 0;
    ma->service[0] = 0;
-   ma->io_mode[0] = 0;
-   ma->flow_ctl[0] = 0;
    ma->oid[0] = 0;
    ma->ip[0] = 0;
    ma->dnode[0] = 0;
+   ma->bus[0] = 0;
+   ma->prt_mode = UNI_MODE;
+   ma->mfp_mode = MLC_MODE;
+   ma->flow_ctl = GUSHER;
+   ma->scan_port = SCAN_PORT0;
    ma->type = PML_DT_UNKNOWN;
    ma->descriptor = -1;
    ma->length = 0;
@@ -1184,6 +1239,10 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
    ma->result = -1;
    ma->timeout = EXCEPTION_TIMEOUT;
    ma->ip_port = 1;
+   ma->pml_result = -1;
+
+   if (buf == NULL)
+      return R_AOK;  /* initialize ma */
 
    i = GetPair(buf, key, value, &tail);
    if (strcasecmp(key, "msg") != 0)
@@ -1236,6 +1295,12 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
       else if (strcasecmp(key, "timeout") == 0)
       {
          ma->timeout = strtol(value, &tail2, 10);
+         if (ma->timeout < 0 || ma->timeout > 120)
+         {
+            syslog(LOG_ERR, "invalid timeout:%d\n", ma->timeout);
+            ret = R_INVALID_TIMEOUT;
+            break;
+         }
       }
       else if (strcasecmp(key, "length") == 0)
       {
@@ -1268,7 +1333,15 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
       }
       else if (strcasecmp(key, "io-mode") == 0)
       {
-         strncpy(ma->io_mode, value, sizeof(ma->io_mode));      /* raw | mlc */
+         ma->prt_mode = strtol(value, &tail2, 10);      /* uni | raw | mlc */
+      }
+      else if (strcasecmp(key, "io-mfp-mode") == 0)
+      {
+         ma->mfp_mode = strtol(value, &tail2, 10);      /* mfc | dot4 */
+      }
+      else if (strcasecmp(key, "io-scan-port") == 0)
+      {
+         ma->scan_port = strtol(value, &tail2, 10);      /* normal | CLJ28xx */
       }
       else if (strcasecmp(key, "result-code") == 0)
       {
@@ -1276,7 +1349,7 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
       }
       else if (strcasecmp(key, "io-control") == 0)
       {
-         strncpy(ma->flow_ctl, value, sizeof(ma->flow_ctl));    /* gusher | miser */
+         ma->flow_ctl = strtol(value, &tail2, 10);     /* gusher | miser */
       }
       else if (strcasecmp(key, "oid") == 0)
       {
@@ -1287,10 +1360,18 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
         //         strncpy(ma->type, value, sizeof(ma->type)); 
          ma->type = strtol(value, &tail2, 10);
       }
+      else if (strcasecmp(key, "pml-result-code") == 0)
+      {
+         ma->pml_result = strtol(value, &tail2, 10);  /* for debug */
+      }
       else if (strcasecmp(key, "hostname") == 0)
       {
          strncpy(ma->ip, value, sizeof(ma->ip));
       }
+      else if (strcasecmp(key, "bus") == 0)
+      {
+         strncpy(ma->bus, value, sizeof(ma->bus));
+      }
       else if (strcasecmp(key, "port") == 0)
       {
          ma->ip_port = strtol(value, &tail2, 10);
@@ -1301,10 +1382,6 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
             break;
          }
       }
-      else if (strcasecmp(key, "type") == 0)
-      {
-         ma->ip_port = strtol(value, &tail2, 10);
-      }
       else if (strcasecmp(key, "device-file") == 0)
       {
          strncpy(ma->dnode, value, sizeof(ma->dnode));
@@ -1326,22 +1403,33 @@ int System::ParseMsg(char *buf, int len, MsgAttributes *ma)
 int System::DeviceCleanUp(SessionAttributes *psa)
 {
    char dummyBuf[255];
-   int i, dummy;
+   int i, n, dummy;
    Device *pD = pDevice[psa->descriptor];
 
    for (i=0; i<MAX_CHANNEL; i++)
    {
       if (psa->channel[i])
       {
-         syslog(LOG_INFO, "channel cleanup id=%d\n", i);
+         syslog(LOG_INFO, "channel cleanup ci=%d\n", i);
          pD->ChannelClose(i, dummyBuf, &dummy);
       }
    }
 
-   syslog(LOG_INFO, "device cleanup id=%d\n", psa->descriptor);
+   syslog(LOG_INFO, "device cleanup uri=%s\n", pD->GetURI());
    pD->Close(dummyBuf, &dummy);
    DelDevice(psa->descriptor);
 
+   /* Check cleanup results. */
+   for (i=0, n=0; i<MAX_DEVICE && n<DeviceCnt; i++)
+   {
+      if (pDevice[i] != NULL)
+      {
+         n++;
+         pD = pDevice[i];
+         syslog(LOG_INFO, "device active clientcnt=%d channelcnt=%d uri=%s\n", pD->GetClientCnt(), pD->GetChannelCnt(), pD->GetURI());
+      }
+   }
+
    return 0;
 }
 
@@ -1349,14 +1437,14 @@ int System::DeviceCleanUp(SessionAttributes *psa)
 //!  Create or re-use device object given the URI.
 /*!
 ******************************************************************************/
-Device *System::NewDevice(char *uri)
+Device *System::NewDevice(MsgAttributes *ma)
 {
    char newDL[255];
    char oldDL[255];
    Device *pD=NULL;
    int i, n;
 
-   if (uri[0] == 0)
+   if (ma->uri[0] == 0)
       return pD;
 
    if (pthread_mutex_lock(&mutex) != 0)
@@ -1371,12 +1459,21 @@ Device *System::NewDevice(char *uri)
       if (pDevice[i] != NULL)
       {
          n++;
-         GetURIDataLink(uri, newDL, sizeof(newDL));
+         GetURIDataLink(ma->uri, newDL, sizeof(newDL));
          GetURIDataLink(pDevice[i]->GetURI(), oldDL, sizeof(oldDL));
          if (strcmp(newDL, oldDL) == 0)
          {
             pD = pDevice[i];   /* same data link */
             pD->SetClientCnt(pD->GetClientCnt()+1);
+
+            /* If existing ProbeDevice/GeneralizeURI set DeviceOpen attributes. */
+            if (strncmp(pD->GetURI(), "ANY?", 4) == 0)
+            {
+               pD->SetPrintMode(ma->prt_mode);            /* io mode for printing */
+               pD->SetMfpMode(ma->mfp_mode);              /* io mode for mfp functions */
+               pD->SetFlowCtl(ma->flow_ctl);              /* flow control for mfp mode */
+               pD->SetScanPort(ma->scan_port);            /* network scan port selection */ 
+            }
             goto bugout;
          }
       }
@@ -1390,12 +1487,48 @@ Device *System::NewDevice(char *uri)
    {
       if (pDevice[i] == NULL)
       {
-         if (strcasestr(uri, "usb/") != NULL)
-            pD = new UsbDevice(this);
-         else
+         if (strcasestr(ma->uri, "hp:/usb") != NULL)
+         {
+            if (ma->prt_mode == UNI_MODE)
+            {
+//               syslog(LOG_INFO, "using uni-di backend for %s\n", ma->uri);
+               pD = new UniUsbDevice(this);
+            }
+            else
+            {
+               pD = new UsbDevice(this);
+               pD->SetPrintMode(ma->prt_mode);            /* io mode for printing */
+               pD->SetMfpMode(ma->mfp_mode);              /* io mode for mfp functions */
+               pD->SetFlowCtl(ma->flow_ctl);              /* flow control for mfp mode */
+            }
+         }
+         else if (strcasestr(ma->uri, "hp:/net") != NULL)
+         {
             pD = new JetDirectDevice(this);
+            pD->SetScanPort(ma->scan_port);            /* network scan port selection */ 
+         }
+         else if (strcasestr(ma->uri, "hp:/par") !=NULL)
+         {
+            if (ma->prt_mode == UNI_MODE)
+            {
+//               syslog(LOG_INFO, "using uni-di backend for %s\n", ma->uri);
+               pD = new UniParDevice(this);
+            }
+            else
+            {
+               pD = new ParDevice(this);
+               pD->SetPrintMode(ma->prt_mode);            /* io mode for printing */
+               pD->SetMfpMode(ma->mfp_mode);              /* io mode for mfp functions */
+               pD->SetFlowCtl(ma->flow_ctl);              /* flow control for mfp mode */
+            }
+         }
+         else
+         {
+            goto bugout;
+         }
+
          pD->SetIndex(i);
-         pD->SetURI(uri);
+         pD->SetURI(ma->uri);
          pDevice[i] = pD;
          DeviceCnt++;
          break;
@@ -1463,7 +1596,13 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    Device *pD;
 
    if ((ret = ParseMsg(recvBuf, rlen, &ma)) != R_AOK)
-      return (sprintf(sendBuf, ERR_MSG, ret));
+   {
+#ifdef HPIOD_DEBUG
+      sysdump(recvBuf, rlen);
+#endif 
+      len = sprintf(sendBuf, ERR_MSG, ret);
+      goto bugout;
+   }
 
 #ifdef HPIOD_DEBUG
    if (ma.length == 0)
@@ -1473,10 +1612,15 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    }
    else
    {
-      syslog(LOG_INFO, "tid:%x %s di=%d ci=%d oid=%s size=%d\n", psa->tid, ma.cmd, ma.descriptor, ma.channel, ma.oid, ma.length);
+      syslog(LOG_INFO, "tid:%x %s di=%d ci=%d oid=%s pml_type=%d size=%d\n", psa->tid, ma.cmd, ma.descriptor, ma.channel, ma.oid, ma.type, ma.length);
       if (ma.length < 64)
-	sysdump(ma.data, ma.length);
+        sysdump(ma.data, ma.length);
    }
+   //   if (ma.descriptor > 0 && pDevice[ma.descriptor])
+   //   {
+   //      pD = pDevice[ma.descriptor];
+   //      syslog(LOG_INFO, "fd=%d clientcnt=%d channelcnt=%d channelmode=%d\n", pD->GetOpenFD(), pD->GetClientCnt(), pD->GetChannelCnt(), pD->GetChannelMode());
+   //   } 
 #endif
 
    if (strcasecmp(ma.cmd, "DeviceID") == 0) 
@@ -1509,7 +1653,7 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    {
       if (psa->descriptor != -1)
          len = sprintf(sendBuf, ERR_MSG, R_INVALID_DEVICE_OPEN); /* allow only one DeviceOpen per session */
-      else if ((pD = NewDevice(ma.uri)) == NULL)
+      else if ((pD = NewDevice(&ma)) == NULL)
          len = sprintf(sendBuf, ERR_MSG, R_INVALID_URI);
       else
       {
@@ -1537,7 +1681,7 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    {
       int channel;
       pD = pDevice[ma.descriptor];
-      len = pD->ChannelOpen(ma.service, ma.io_mode, ma.flow_ctl, &channel, sendBuf, &ret);
+      len = pD->ChannelOpen(ma.service, &channel, sendBuf, &ret);
       if (ret == R_AOK)
          psa->channel[channel] = 1;   /* track channel descriptor for session clean up */
    }
@@ -1549,7 +1693,7 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    }
    else if (strcasecmp(ma.cmd, "ProbeDevices") == 0)
    {
-      len = ProbeDevices(sendBuf);       
+      len = ProbeDevices(sendBuf, ma.bus);       
    }
    else if (strcasecmp(ma.cmd, "DeviceFile") == 0)
    {
@@ -1559,8 +1703,10 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    {
       if (ma.ip[0])
          len = MakeUriFromIP(ma.ip, ma.ip_port, sendBuf);
+      else if (strcasecmp(ma.bus, "par") == 0) 
+         len = MakeUriFromPar(ma.dnode, sendBuf);    
       else 
-         len = MakeUriFromDevice(ma.dnode, sendBuf);     
+         len = MakeUriFromUsb(ma.dnode, sendBuf);    
    }
    else
    {
@@ -1569,6 +1715,8 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
       len = sprintf(sendBuf, ERR_MSG, R_INVALID_MESSAGE);
    }
 
+bugout:
+
 #ifdef HPIOD_DEBUG
    ParseMsg(sendBuf, len, &ma);
    if (ma.length == 0)
@@ -1577,9 +1725,9 @@ int System::ExecuteMsg(SessionAttributes *psa, char *recvBuf, int rlen, char *se
    }
    else
    {
-      syslog(LOG_INFO, "-tid:%x %s di=%d ci=%d size=%d\n", psa->tid, ma.cmd, ma.descriptor, ma.channel, ma.length);
+      syslog(LOG_INFO, "-tid:%x %s di=%d ci=%d result=%d pml_result=%d pml_type=%d size=%d\n", psa->tid, ma.cmd, ma.descriptor, ma.channel, ma.result, ma.pml_result, ma.type, ma.length);
       if (ma.length < 64)
-	sysdump(ma.data, ma.length);
+        sysdump(ma.data, ma.length);
    }
 #endif
 
