@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
-# $Revision: 1.15 $ 
-# $Date: 2005/03/21 17:38:49 $
+# $Revision: 1.17 $
+# $Date: 2005/07/21 17:31:38 $
 # $Author: dwelch $
 #
 # (c) Copyright 2003-2004 Hewlett-Packard Development Company, L.P.
@@ -43,34 +43,21 @@ from base.g import *
 from base import device, service, utils #, exif
 from prnt import cups
 from pcard import photocard
-   
+
 def usage():
-    formatter = utils.TextFormatter( 
-                (
-                    {'width': 38, 'margin' : 2},
-                    {'width': 38, 'margin' : 2},
-                )
-            )
+    formatter = utils.usage_formatter()
+    log.info(utils.bold("""\nUsage: hp-photo [PRINTER|DEVICE-URI] [OPTIONS]\n\n""" ))
+    log.info(utils.bold( "[PRINTER|DEVICE-URI] (**See NOTES)" ) )
+    utils.usage_device(formatter)
+    utils.usage_printer(formatter, True)
+    utils.usage_options()
+    utils.usage_bus(formatter)
+    utils.usage_logging(formatter)
+    utils.usage_help(formatter, True)
+    utils.usage_notes()
+    log.info("\n\tUse 'help' command at the pcard:> prompt for command help.")
+    sys.exit(0)
 
-    log.info( utils.TextFormatter.bold( """\nUsage: hp-photo [PRINTER|DEVICE-URI] [OPTIONS]\n\n""") )
-    
-    log.info( formatter.compose( ( utils.TextFormatter.bold("[PRINTER|DEVICE-URI]"), "" ) ) )
-    log.info( formatter.compose( ( "(**See NOTES 1&2)",                     "" ) ) )
-    log.info( formatter.compose( ( "To specify a CUPS printer:",           "-p<printer> or --printer=<printer>" ) ) )
-    log.info( formatter.compose( ( "To specify a device-URI:",             "-d<device-uri> or --device=<device-uri>" ), True ) )
-    log.info( formatter.compose( ( utils.TextFormatter.bold("[OPTIONS]"),            "" ) ) )
-    log.info( formatter.compose( ( "Set the logging level:",               "-l<level> or --logging=<level>" ) ) )
-    log.info( formatter.compose( ( "",                                     "<level>: none, info*, error, warn, debug (*default)" ) ) )
-    log.info( formatter.compose( ( "Bus to probe (interactive mode only):","-b<bus> or --bus=<bus>" ) ) )
-    log.info( formatter.compose( ( "",                                     "<bus>: cups*, usb, net, bt, fw, par (*default) (Note: net, bt, fw, and par not supported)" ) ) )
-    log.info( formatter.compose( ( "This help information:",               "-h or --help" ), True ) )
-
-    log.info(  """Examples:\n\nAccess photo cards on a CUPS printer named "hp5550":\n   hp-photo -php5550\n\n""" \
-               """Access photo cards on printer with a URI of "hp:/usb/DESKJET_990C?serial=12345":\n   hp-photo -dhp:/usb/DESKJET_990C?serial=12345\n\n"""\
-               """**NOTES: 1. If device or printer is not specified, the local device bus\n""" \
-               """            is probed and the program enters interactive mode.\n""" \
-               """         2. If -p* is specified, the default CUPS printer will be used.\n""" )
-               
 
 ## Console class (from ASPN Python Cookbook)
 ## Author:   James Thiele
@@ -83,14 +70,14 @@ class Console(cmd.Cmd):
 
     def __init__(self, pc ):
         cmd.Cmd.__init__(self)
-        self.intro  = "Type 'help' for a list of commands. Type 'exit' to quit."  
+        self.intro  = "Type 'help' for a list of commands. Type 'exit' to quit."
         self.pc = pc
         disk_info = self.pc.info()
         pc.write_protect = disk_info[8]
         if pc.write_protect:
             log.warning( "Photo card is write protected." )
         self.prompt = utils.bold( "pcard: %s > " % self.pc.pwd() )
-    
+
     ## Command definitions ##
     def do_hist(self, args):
         """Print a list of commands that have been entered"""
@@ -99,7 +86,7 @@ class Console(cmd.Cmd):
     def do_exit(self, args):
         """Exits from the console"""
         return -1
-        
+
     def do_quit( self, args ):
         """Exits from the console"""
         return -1
@@ -148,74 +135,74 @@ class Console(cmd.Cmd):
         """
         return stop
 
-    def emptyline(self):    
+    def emptyline(self):
         """Do nothing on empty input line"""
         pass
 
-    def default(self, line):       
+    def default(self, line):
         print utils.bold( "ERROR: Unrecognized command. Use 'help' to list commands." )
-    
+
     def do_ldir( self, args ):
         """ List local directory contents."""
         os.system( 'ls -l' )
-    
+
     def do_lls( self, args ):
         """ List local directory contents."""
         os.system( 'ls -l' )
-    
+
     def do_dir( self, args ):
         """Synonym for the ls command."""
         return self.do_ls( args )
-    
+
     def do_ls( self, args ):
         """List photo card directory contents."""
         args = args.strip().lower()
-        files = self.pc.ls( True, args ) 
-        
+        files = self.pc.ls( True, args )
+
         if 1:
-        
+
             total_size = 0
-            formatter = utils.TextFormatter( 
+            formatter = utils.TextFormatter(
                     (
                         {'width': 14, 'margin' : 2},
                         {'width': 12, 'margin' : 2, 'alignment' : utils.TextFormatter.RIGHT },
                         {'width': 30, 'margin' : 2},
                     )
                 )
-            
+
             print
             print utils.bold( formatter.compose( ("Name", "Size", "Type" ) ) )
-            
+
             num_files = 0
             for d in self.pc.current_directories():
                 if d[0] in ('.', '..'):
-                    print formatter.compose( ( d[0], "", "directory" ) ) 
+                    print formatter.compose( ( d[0], "", "directory" ) )
                 else:
-                    print formatter.compose( ( d[0] + "/", "", "directory" ) ) 
-            
+                    print formatter.compose( ( d[0] + "/", "", "directory" ) )
+
             for f in self.pc.current_files():
                 print formatter.compose( ( f[0], utils.format_bytes(f[2]), self.pc.classify_file( f[0] ) ) )
                 num_files += 1
                 total_size += f[2]
-            
+
             print utils.bold( "% d files, %s" % ( num_files, utils.format_bytes( total_size, True ) ) )
-            
-            
+
+
     def do_df( self, args ):
         """Display free space on photo card.
         Options:
         -h\tDisplay in human readable format
         """
         freespace = self.pc.df()
-        
+
         if args.strip().lower() == '-h':
             fs = utils.format_bytes( freespace )
         else:
             fs = utils.commafy(freespace)
-        
+
         print "Freespace = %s Bytes" % fs
-            
-    
+
+
     def do_cp( self, args, remove_after_copy=False ):
         """Copy files from photo card to current local directory.
         Usage:
@@ -225,16 +212,16 @@ class Console(cmd.Cmd):
         \tcp *.jpg *.gif thumbs.db
         """
         args = args.strip().lower()
-        
+
         matched_files = self.pc.match_files( args )
-        
-        if len( matched_files ) == 0: 
+
+        if len( matched_files ) == 0:
             print "ERROR: File(s) not found."
         else:
             total, delta = self.pc.cp_multiple( matched_files, remove_after_copy, self.cp_status_callback, self.rm_status_callback )
-            
+
             print utils.bold( "\n%s transfered in %d sec (%d KB/sec)" % ( utils.format_bytes( total ), delta, (total/1024)/(delta) ) )
-            
+
     def do_unload( self, args ):
         """Unload all image files from photocard to current local directory.
         Note:
@@ -244,89 +231,88 @@ class Console(cmd.Cmd):
         -p\tPrint unload list but do not copy or remove files"""
         args = args.lower().strip().split()
         dont_remove = False
-        if '-x' in args: 
+        if '-x' in args:
             if self.pc.write_protect:
                 log.error( "Photo card is write protected. -x not allowed." )
                 return
             else:
                 dont_remove = True
-            
-        
+
+
         unload_list = self.pc.get_unload_list()
-        #print unload_list
         print
-        
+
         if len( unload_list ) > 0:
             if '-p' in args:
-                
+
                 max_len = 0
                 for u in unload_list:
                     max_len = max( max_len, len(u[0]) )
-                
-                formatter = utils.TextFormatter( 
+
+                formatter = utils.TextFormatter(
                         (
                             {'width': max_len+2, 'margin' : 2},
                             {'width': 12, 'margin' : 2, 'alignment' : utils.TextFormatter.RIGHT },
                             {'width': 12, 'margin' : 2},
                         )
                     )
-                
+
                 print
                 print utils.bold( formatter.compose( ("Name", "Size", "Type" ) ) )
-                
+
                 total = 0
                 for u in unload_list:
-                     print formatter.compose( ( '%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % ( u[2], u[3] ) )) 
+                     print formatter.compose( ( '%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % ( u[2], u[3] ) ))
                      total += u[1]
-                     
-                
+
+
                 print utils.bold( "Found %d files to unload, %s" % ( len(unload_list), utils.format_bytes( total, True ) ) )
             else:
                 print utils.bold( "Unloading %d files..." % len(unload_list) )
                 total, delta, was_cancelled = self.pc.unload( unload_list, self.cp_status_callback, self.rm_status_callback, dont_remove )
                 print utils.bold( "\n%s unloaded in %d sec (%d KB/sec)" % ( utils.format_bytes( total ), delta, (total/1024)/delta ) )
-            
+
         else:
             print "No image, audio, or video files found."
-            
-            
+
+
     def cp_status_callback( self, src, trg, size ):
         print "Copying %s to %s (%s)..." % ( src, trg, utils.format_bytes(size) )
-        
+
     def rm_status_callback( self, src ):
         print "Removing %s..." % src
-    
-        
-        
+
+
+
     def do_rm( self, args ):
         """Remove files from photo card."""
         if self.pc.write_protect:
             log.error( "Photo card is write protected. rm not allowed." )
             return
-        
+
         args = args.strip().lower()
-        
+
         matched_files = self.pc.match_files( args )
-        
-        if len( matched_files ) == 0: 
+
+        if len( matched_files ) == 0:
             print "ERROR: File(s) not found."
         else:
             for f in matched_files:
                 self.pc.rm( f, False )
-                
+
         self.pc.ls()
-        
+
     def do_mv( self, args ):
         """Move files off photocard"""
         if self.pc.write_protect:
             log.error( "Photo card is write protected. mv not allowed." )
             return
         self.do_cp( args, True )
-        
+
     def do_lpwd( self, args ):
         """Print name of local current/working directory."""
         print os.getcwd()
-        
+
     def do_lcd( self, args ):
         """Change current local working directory."""
         try:
@@ -334,13 +320,13 @@ class Console(cmd.Cmd):
         except OSError:
             print utils.bold( "ERROR: Directory not found." )
         print os.getcwd()
-        
+
     def do_pwd( self, args ):
         """Print name of photo card current/working directory
         Usage:
         \t>pwd"""
         print self.pc.pwd()
-        
+
     def do_cd( self, args ):
         """Change current working directory on photo card.
         Note:
@@ -349,39 +335,39 @@ class Console(cmd.Cmd):
         \tcd <directory>
         """
         args = args.lower().strip()
-        
+
         if args == '..':
             if self.pc.pwd() != '/':
                 self.pc.cdup()
-        
+
         elif args == '.':
             pass
-        
+
         elif args == '/':
             self.pc.cd('/')
-        
+
         else:
             matched_dirs = self.pc.match_dirs( args )
-            
+
             if len( matched_dirs ) == 0:
-                print "Directory not found" 
-            
+                print "Directory not found"
+
             elif len( matched_dirs ) > 1:
                 print "Pattern matches more than one directory"
-            
+
             else:
                 self.pc.cd( matched_dirs[0] )
-        
+
         self.prompt = utils.bold( "pcard: %s > " % self.pc.pwd() )
 
     def do_cdup( self, args ):
         """Change to parent directory."""
         self.do_cd( '..' )
-    
+
     #def complete_cd( self, text, line, begidx, endidx ):
     #    print text, line, begidx, endidx
     #    #return "XXX"
-        
+
     def do_cache( self, args ):
         """Display current cache entries, or turn cache on/off.
         Usage:
@@ -390,28 +376,28 @@ class Console(cmd.Cmd):
         \tTurn off: cache off
         """
         args = args.strip().lower()
-        
+
         if args == 'on':
             self.pc.cache_control(True)
-        
+
         elif args == 'off':
             self.pc.cache_control(False)
-        
+
         else:
             if self.pc.cache_state():
                 cache_info = self.pc.cache_info()
-                
+
                 t = cache_info.keys()
                 t.sort()
                 print
                 for s in t:
                     print "sector %d (%d hits)" % ( s, cache_info[s] )
-                    
+
                 print utils.bold( "Total cache usage: %s (%s maximum)" % (utils.format_bytes(len(t)*512), utils.format_bytes(photocard.MAX_CACHE * 512)))
                 print utils.bold( "Total cache sectors: %s of %s" % (utils.commafy(len(t)), utils.commafy(photocard.MAX_CACHE)))
             else:
                 print "Cache is off."
-        
+
     def do_sector( self, args ):
         """Display sector data.
         Usage:
@@ -424,26 +410,26 @@ class Console(cmd.Cmd):
         except ValueError:
             print "Sector must be specified as a number"
             return
-        
+
         if self.pc.cache_check( sector ) > 0:
             print "Cached sector"
-        
+
         print repr( self.pc.sector( sector ) )
-        
-        
+
+
     def do_tree( self, args ):
         """Display photo card directory tree."""
         tree = self.pc.tree( )
         print
         self.print_tree( tree )
-        
+
     def print_tree( self, tree, level=0 ):
         for d in tree:
             if type(tree[d]) == type({}):
                 print ''.join( [ ' '*level*4, d, '/' ] )
                 self.print_tree( tree[d], level+1 )
-    
-        
+
+
     def do_reset( self, args ):
         """Reset the cache."""
         self.pc.cache_reset()
@@ -474,30 +460,30 @@ class Console(cmd.Cmd):
         \tdisplay <filename>"""
         args = args.strip().lower()
         matched_files = self.pc.match_files( args )
-        
+
         if len( matched_files ) == 1:
-            
+
             typ = self.pc.classify_file( args ).split('/')[0]
-            
+
             if typ == 'image':
                 fd, temp_name = utils.make_temp_file()
                 self.pc.cp( args, temp_name )
                 os.system( 'display %s' % temp_name )
                 os.remove( temp_name )
-                
+
             else:
                 print "File is not an image."
-        
+
         elif len( matched_files ) == 0:
             print "File not found."
-        
+
         else:
             print "Only one file at a time may be specified for display."
-            
+
     def do_show( self, args ):
         """Synonym for the display command."""
         self.do_display( args )
-        
+
     def do_thumbnail( self, args ):
         """Display an embedded thumbnail image with ImageMagick.
         Note:
@@ -506,17 +492,17 @@ class Console(cmd.Cmd):
         \tthumbnail <filename>"""
         args = args.strip().lower()
         matched_files = self.pc.match_files( args )
-        
+
         if len( matched_files ) == 1:
             typ, subtyp = self.pc.classify_file( args ).split('/')
             #print "'%s' '%s'" % (typ, subtyp)
-            
+
             if typ == 'image' and subtyp in ( 'jpeg', 'tiff' ):
                 exif_info = self.pc.get_exif( args )
-                
+
                 dir_name, file_name=os.path.split(args)
                 photo_name, photo_ext=os.path.splitext(args)
-                
+
                 if 'JPEGThumbnail' in exif_info:
                     #print "JPEG thumbnail found."
                     temp_file_fd, temp_file_name = utils.make_temp_file()
@@ -524,7 +510,7 @@ class Console(cmd.Cmd):
                     open(temp_file_name, 'wb').write(exif_info['JPEGThumbnail'])
                     os.system( 'display %s' % temp_file_name )
                     os.remove( temp_file_name )
-                
+
                 elif 'TIFFThumbnail' in exif_info:
                     #print "TIFF thumbnail found."
                     #thumb_name = os.path.join( os.getcwd(), photo_name ) + '_thumb.tif'
@@ -532,46 +518,46 @@ class Console(cmd.Cmd):
                     open(temp_file_name, 'wb').write(exif_info['TIFFThumbnail'])
                     os.system( 'display %s' % temp_file_name )
                     os.remove( temp_file_name )
-                
+
                 else:
                     print "No thumbnail found."
-                
+
             else:
                 print "Incorrect file type for thumbnail."
-        
+
         elif len( matched_files ) == 0:
             print "File not found."
         else:
             print "Only one file at a time may be specified for thumbnail display."
-        
+
     def do_thumb( self, args):
         """Synonym for the thumbnail command."""
         self.do_thumbnail( args )
-        
+
     def do_exif( self, args ):
         """Display EXIF info for file.
         Usage:
         \texif <filename>"""
         args = args.strip().lower()
         matched_files = self.pc.match_files( args )
-        
+
         if len( matched_files ) == 1:
             typ, subtyp = self.pc.classify_file( args ).split('/')
             #print "'%s' '%s'" % (typ, subtyp)
-            
+
             if typ == 'image' and subtyp in ( 'jpeg', 'tiff' ):
                 exif_info = self.pc.get_exif( args )
-                
-                formatter = utils.TextFormatter( 
+
+                formatter = utils.TextFormatter(
                         (
                             {'width': 40, 'margin' : 2},
                             {'width': 40, 'margin' : 2},
                         )
                     )
-                
+
                 print
                 print utils.bold( formatter.compose( ("Tag", "Value" ) ) )
-                
+
                 ee = exif_info.keys()
                 ee.sort()
                 for e in ee:
@@ -582,68 +568,58 @@ class Console(cmd.Cmd):
                         #    print formatter.compose( ( e, ''.join( [ chr(x) for x in exif_info[e].values if chr(x) in string.printable ] ) ) )
             else:
                 print "Incorrect file type for thumbnail."
-        
+
         elif len( matched_files ) == 0:
             print "File not found."
         else:
             print "Only one file at a time may be specified for thumbnail display."
-            
+
     def do_info( self, args ):
         """Synonym for the exif command."""
         self.do_exif( args )
-        
-        
-        
-        
+
+
+
+
 utils.log_title( 'Photo Card Access Utility', _VERSION )
 
 try:
-    opts, args = getopt.getopt( sys.argv[1:], 'p:d:hb:l:', 
-                               [ 'printer=', 'device=', 'help', 'bus=', 'logging=' ] ) 
+    opts, args = getopt.getopt( sys.argv[1:], 'p:d:hb:l:',
+                               [ 'printer=', 'device=', 'help', 'bus=', 'logging=' ] )
 except getopt.GetoptError:
     usage()
-    sys.exit(0)
-    
+
 printer_name = None
-device_uri = None    
-bus = 'usb,cups'
-log_level = 'info'
+device_uri = None
+bus = device.DEFAULT_PROBE_BUS
+log_level = logger.DEFAULT_LOG_LEVEL
 
 for o, a in opts:
     if o in ( '-h', '--help' ):
         usage()
-        sys.exit(0)
-    
+
     elif o in ( '-p', '--printer' ):
         printer_name = a
-    
+
     elif o in ( '-d', '--device' ):
         device_uri = a
-        
+
     elif o in ( '-b', '--bus' ):
         bus = a.lower().strip()
-        
+
     elif o in ( '-l', '--logging' ):
         log_level = a.lower().strip()
 
-    
-if not log_level in ( 'info', 'warn', 'error', 'debug' ):
-    log.error( "Invalid logging level." )
-    sys.exit(0)
-    
-log.set_level( log_level )   
-   
-for x in bus.split(','):
-    bb = x.lower().strip()
-    #if not bb in ( 'usb', 'net', 'bt', 'fw' ):
-    if bb not in ( 'usb', 'cups', 'net' ):
-        log.error( "Invalid bus name: %s" % bb )
-        usage()
-        sys.exit(0)
-        
+
+if not device.validateBusList(bus):
+    usage()
+
+if not log.set_level( log_level ):
+    usage()
+
 if device_uri and printer_name:
     log.error( "You may not specify both a printer (-p) and a device (-d)." )
-    sys.exit(0)
+    usage()
 
 if printer_name:
     printer_list = cups.getPrinters()
@@ -651,7 +627,7 @@ if printer_name:
     for p in printer_list:
         if p.name == printer_name:
             found = True
-    
+
     if not found:
         log.error( "Unknown printer name: %s" % printer_name )
         sys.exit(0)
@@ -659,7 +635,7 @@ if printer_name:
 
 if not device_uri and not printer_name:
     try:
-        device_uri = utils.getInteractiveDeviceURI( bus, 'pcard' )
+        device_uri = device.getInteractiveDeviceURI( bus, 'pcard' )
         if device_uri is None:
             sys.exit(0)
     except Error:
@@ -672,19 +648,23 @@ pc.set_callback( update_spinner )
 if pc.device.device_uri is None and printer_name:
     log.error( "Printer '%s' not found." % printer_name )
     sys.exit(0)
-    
+
 if pc.device.device_uri is None and device_uri:
     log.error( "Malformed/invalid device-uri: %s" % device_uri )
     sys.exit(0)
-    
+
+pc.device.sendEvent( EVENT_START_PCARD_JOB, 'event', 0,
+    prop.username, pc.device.device_uri )
 
 try:
     pc.mount()
 except Error:
     log.error( "Unable to mount photo card on device. Check that device is powered on and photo card is correctly inserted." )
     pc.umount()
+    pc.device.sendEvent( EVENT_PCARD_UNABLE_TO_MOUNT, 'error', 0, prop.username,
+        pc.device.device_uri )
     sys.exit(0)
-    
+
 log.info( utils.bold("\nPhotocard on device %s mounted" % pc.device.device_uri ) )
 log.info( utils.bold("DO NOT REMOVE PHOTO CARD UNTIL YOU EXIT THIS PROGRAM") )
 
@@ -697,9 +677,12 @@ try:
 finally:
     pc.umount()
 
+pc.device.sendEvent( EVENT_END_PCARD_JOB, 'event', 0, prop.username,
+    pc.device.device_uri )
+
 log.info( "Done." )
 
 
 
 
-    
+
