@@ -1,7 +1,7 @@
 /*
 cupsext - Python extension class for CUPS 1.1+
- 
- (c) Copyright 2003-2004 Hewlett-Packard Development Company, L.P.
+
+ (c) Copyright 2003-2005 Hewlett-Packard Development Company, L.P.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -20,25 +20,25 @@ cupsext - Python extension class for CUPS 1.1+
 
 Portions based on:
     "lpadmin" command for the Common UNIX Printing System (CUPS).
- 
+
     Copyright 1997-2003 by Easy Software Products.
- 
+
     These coded instructions, statements, and computer programs are the
     property of Easy Software Products and are protected by Federal
     copyright law.  Distribution and use rights are outlined in the file
     "LICENSE.txt" which should have been included with this file.  If this
     file is missing or damaged please contact Easy Software Products
     at:
- 
+
         Attn: CUPS Licensing Information
         Easy Software Products
         44141 Airport View Drive, Suite 204
         Hollywood, Maryland 20636-3111 USA
- 
+
         Voice: (301) 373-9603
         EMail: cups-info@cups.org
           WWW: http://www.cups.org
- 
+
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
@@ -119,6 +119,7 @@ typedef struct
     PyObject * name;
     PyObject * location;
     PyObject * makemodel;
+    PyObject * info;
     /*PyObject * ppd_name;*/
     int state;
 } printer_Object;
@@ -126,18 +127,19 @@ typedef struct
 
 static void printer_dealloc( printer_Object * self )
 {
-    
+
     Py_XDECREF(self->name);
     Py_XDECREF(self->device_uri);
     Py_XDECREF(self->printer_uri);
     Py_XDECREF(self->location);
     Py_XDECREF(self->makemodel);
+    Py_XDECREF(self->info);
     /*Py_XDECREF(self->ppd_name);*/
     PyObject_DEL( self );
 }
 
 
-static PyMemberDef printer_members[] = 
+static PyMemberDef printer_members[] =
 {
     { "device_uri", T_OBJECT_EX, offsetof( printer_Object, device_uri ), 0, "Device URI (device-uri)" },
     /*{ "ppd_name", T_OBJECT_EX, offsetof( printer_Object, ppd_name ), 0, "PPD Name (ppd-name)" },*/
@@ -146,15 +148,16 @@ static PyMemberDef printer_members[] =
     { "location", T_OBJECT_EX, offsetof( printer_Object, location ), 0, "Location (printer-location)" },
     { "makemodel", T_OBJECT_EX, offsetof( printer_Object, makemodel ), 0, "Make and model (printer-make-and-model)" },
     { "state", T_INT, offsetof( printer_Object, state ), 0, "State (printer-state)" },
+    { "info", T_OBJECT_EX, offsetof( printer_Object, info ), 0, "Info/description (printer-info)" },
     {0}
 };
 
-static PyTypeObject printer_Type = 
+static PyTypeObject printer_Type =
 {
-    PyObject_HEAD_INIT( &PyType_Type ) 
+    PyObject_HEAD_INIT( &PyType_Type )
     0,                             /* ob_size */
     "cupsext.Printer",           /* tp_name */
-    sizeof( printer_Object ),      /* tp_basicsize */    
+    sizeof( printer_Object ),      /* tp_basicsize */
     0,                             /* tp_itemsize */
     (destructor)printer_dealloc,   /* tp_dealloc */
     0,                             /* tp_print */
@@ -191,20 +194,21 @@ static PyTypeObject printer_Type =
     0,                             /* tp_alloc */
     0,                             /* tp_new */
 };
-    
 
 
 
-static PyObject * _newPrinter( char * device_uri, 
-                                char * name,
-                            char * printer_uri, 
-                                                    char * location, 
-                            char * makemodel, 
-                            int    state
+
+static PyObject * _newPrinter( char * device_uri,
+                               char * name,
+                               char * printer_uri,
+                               char * location,
+                               char * makemodel,
+                               char * info,
+                               int    state
                             /*char * ppd_name*/ )
 {
     printer_Object * self = PyObject_New( printer_Object, &printer_Type );
-    if ( !self ) 
+    if ( !self )
         return NULL;
     if ( device_uri != NULL )
         self->device_uri = Py_BuildValue( "s", device_uri );
@@ -216,11 +220,13 @@ static PyObject * _newPrinter( char * device_uri,
         self->location = Py_BuildValue( "s", location );
     if ( makemodel != NULL )
         self->makemodel = Py_BuildValue( "s", makemodel );
+    if (info != NULL )
+        self->info = Py_BuildValue( "s", info );
     /*if ( ppd_name != NULL )
         self->ppd_name = Py_BuildValue( "s", ppd_name );*/
     self->state = state;
     return (PyObject *)self;
-}   
+}
 
 static PyObject * newPrinter( PyObject * self, PyObject * args, PyObject * kwargs )
 {
@@ -230,23 +236,23 @@ static PyObject * newPrinter( PyObject * self, PyObject * args, PyObject * kwarg
     char * makemodel = "";
     int state = 0;
     char * printer_uri = "";
+    char * info = "";
     /*char * ppd_name = "";*/
-        
-    char * kwds[] = { "device_uri", "name", "printer_uri", "location", 
-                      "makemodel", "state", /*"ppd_name",*/ NULL };
-        
-    if (!PyArg_ParseTupleAndKeywords( args, kwargs, "zz|zzzi", kwds, 
-                                      &device_uri, &name, &printer_uri, 
-                      &location, &makemodel, &state 
-                      /*&ppd_name*/ ))
-        return NULL;        
-    
-    return _newPrinter( device_uri, printer_uri, name, location, makemodel, state /*ppd_name*/ );
-}   
 
-PyObject * getPrinters( PyObject * self, PyObject * args ) 
+    char * kwds[] = { "device_uri", "name", "printer_uri", "location",
+                      "makemodel", "info", "state", /*"ppd_name",*/ NULL };
+
+    if (!PyArg_ParseTupleAndKeywords( args, kwargs, "zz|zzzzi", kwds,
+                                      &device_uri, &name, &printer_uri,
+                                      &location, &makemodel, &info, &state
+                                      /*&ppd_name*/ ))
+        return NULL;
+
+    return _newPrinter( device_uri, printer_uri, name, location, makemodel, info, state /*ppd_name*/ );
+}
+
+PyObject * getPrinters( PyObject * self, PyObject * args )
 {
-    char buf[1024];
     http_t *http=NULL;     /* HTTP object */
     ipp_t *request=NULL;  /* IPP request object */
     ipp_t *response=NULL; /* IPP response object */
@@ -274,7 +280,7 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
     /* Assemble the IPP request */
     request = ippNew();
     language = cupsLangDefault();
-    
+
     request->request.op.operation_id = CUPS_GET_PRINTERS;
     request->request.any.request_id  = 1;
 
@@ -297,9 +303,9 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
     int max_count = 0;
 
     for ( attr = ippFindAttribute(response, "printer-name", IPP_TAG_NAME),
-            max_count = 0; 
+            max_count = 0;
         attr != NULL;
-        attr = ippFindNextAttribute(response, "printer-name", IPP_TAG_NAME), 
+        attr = ippFindNextAttribute(response, "printer-name", IPP_TAG_NAME),
             max_count ++);
 
 
@@ -341,7 +347,7 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
                 sprintf( buf, "print '%s=%s'", attr->name, attr->values[0].string.text );
                 PyRun_SimpleString( buf );
             }*/
-            
+
             if (strcmp(attr->name, "printer-name") == 0 &&
                 attr->value_tag == IPP_TAG_NAME)
                 name = attr->values[0].string.text;
@@ -353,7 +359,7 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
             else if (strcmp(attr->name, "printer-uri-supported") == 0 &&
                 attr->value_tag == IPP_TAG_URI)
                 printer_uri = attr->values[0].string.text;
-            
+
             else if (strcmp(attr->name, "printer-info") == 0 &&
                 attr->value_tag == IPP_TAG_TEXT)
                 info = attr->values[0].string.text;
@@ -369,11 +375,11 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
             else if (strcmp(attr->name, "printer-state") == 0 &&
                 attr->value_tag == IPP_TAG_ENUM)
                 state = (ipp_pstate_t)attr->values[0].integer;
-                
+
             /*else if (strcmp(attr->name, "ppd-name") == 0 &&
                 attr->value_tag == IPP_TAG_NAME)
                 ppd_name = attr->values[0].string.text;*/
-                
+
 
             attr = attr->next;
         }
@@ -384,17 +390,17 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
                 break;
             else
                 continue;
-        }  
+        }
 
         printer_Object * printer;
         printer = (printer_Object *)_newPrinter( device_uri, name, printer_uri, location, make_model,
-        state/*, ppd_name*/ ); 
+                                                 info, state/*, ppd_name*/ );
         PyList_SET_ITEM( printer_list, i, (PyObject *)printer );
 
         i++;
 
         if (attr == NULL)
-            break;  
+            break;
     }
 
     return printer_list;
@@ -402,31 +408,30 @@ PyObject * getPrinters( PyObject * self, PyObject * args )
 abort:
     if (response != NULL)
         ippDelete(response);
-        
+
     if (http != NULL)
         httpClose(http);
-    
+
     printer_list = PyList_New( 0 );
     return printer_list;
 }
 
 
-PyObject *  addPrinter( PyObject * self, PyObject * args ) 
+PyObject *  addPrinter( PyObject * self, PyObject * args )
 {
     //char buf[1024];
     ipp_status_t status;
     http_t *http=NULL;     /* HTTP object */
     ipp_t *request=NULL;  /* IPP request object */
     ipp_t *response=NULL; /* IPP response object */
-    ipp_attribute_t *attr;     /* Current IPP attribute */
     cups_lang_t * language;
     int r;
-    char printer_uri[HTTP_MAX_URI];  
+    char printer_uri[HTTP_MAX_URI];
     char * name, * device_uri, *location, *ppd_file, * info;
     const char * status_str = "successful-ok";
 
 
-    if ( !PyArg_ParseTuple( args, "zzzzz", 
+    if ( !PyArg_ParseTuple( args, "zzzzz",
                             &name, // name of printer
                             &device_uri, // DeviceURI (e.g., hp:/usb/PSC_2200_Series?serial=0000000010)
                             &location, // location of printer
@@ -446,12 +451,12 @@ PyObject *  addPrinter( PyObject * self, PyObject * args )
         goto abort;
     }
 
-    
+
     sprintf( printer_uri, "ipp://localhost/printers/%s", name );
-    
+
     if (info == NULL )
         strcpy( info, name );
-    
+
      /* Connect to the HTTP server */
     if ((http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption())) == NULL)
     {
@@ -463,7 +468,7 @@ PyObject *  addPrinter( PyObject * self, PyObject * args )
     /* Assemble the IPP request */
     request = ippNew();
     language = cupsLangDefault();
-    
+
     request->request.op.operation_id = CUPS_ADD_PRINTER;
     request->request.any.request_id  = 1;
 
@@ -472,31 +477,31 @@ PyObject *  addPrinter( PyObject * self, PyObject * args )
 
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
         "attributes-natural-language", NULL, language->language );
-    
+
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
         "printer-uri", NULL, printer_uri );
 
     ippAddInteger(request, IPP_TAG_PRINTER, IPP_TAG_ENUM,
         "printer-state", IPP_PRINTER_IDLE );
-        
+
     ippAddBoolean( request, IPP_TAG_PRINTER, "printer-is-accepting-jobs", 1 );
-    
+
     ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_URI, "device-uri", NULL,
-                 device_uri );  
-         
+                 device_uri );
+
         ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_TEXT, "printer-info", NULL,
                info );
-           
+
         ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_TEXT, "printer-location", NULL,
                location );
-    
+
     /* Send the request and get a response. */
     if ((response = cupsDoFileRequest(http, request, "/admin/", ppd_file )) == NULL)
     {
         status = cupsLastError();
         r = 0;
     }
-    else    
+    else
     {
         status = response->request.status.status_code;
         ippDelete( response );
@@ -506,7 +511,7 @@ PyObject *  addPrinter( PyObject * self, PyObject * args )
     status_str = ippErrorString( status );
 
 abort:
-        
+
     if (http != NULL)
         httpClose(http);
 
@@ -517,7 +522,7 @@ abort:
 /*
  * 'delPrinter()' - Delete a printer from the system...
  */
-PyObject * delPrinter( PyObject * self, PyObject * args ) 
+PyObject * delPrinter( PyObject * self, PyObject * args )
 {
   ipp_t         *request = NULL,                /* IPP Request */
                 *response = NULL;                /* IPP Response */
@@ -526,8 +531,8 @@ PyObject * delPrinter( PyObject * self, PyObject * args )
   char * name;
   http_t *http = NULL;     /* HTTP object */
   int r = 0;
- 
-  if ( !PyArg_ParseTuple( args, "z", 
+
+  if ( !PyArg_ParseTuple( args, "z",
                           &name ) ) // name of printer
   {
       goto abort;
@@ -573,7 +578,7 @@ PyObject * delPrinter( PyObject * self, PyObject * args )
   * Do the request and get back a response...
   */
   response = cupsDoRequest(http, request, "/admin/");
-  
+
   if ( (response != NULL) && (response->request.status.status_code <= IPP_OK_CONFLICT) )
   {
     r = 1;
@@ -591,7 +596,7 @@ abort:
  * 'setDefaultPrinter()' - Set the default printing destination.
  */
 
-PyObject * setDefaultPrinter( PyObject * self, PyObject * args ) 
+PyObject * setDefaultPrinter( PyObject * self, PyObject * args )
 
 {
   char          uri[HTTP_MAX_URI];        /* URI for printer/class */
@@ -601,8 +606,8 @@ PyObject * setDefaultPrinter( PyObject * self, PyObject * args )
   char * name;
   http_t *http = NULL;     /* HTTP object */
   int r = 0;
- 
-  if ( !PyArg_ParseTuple( args, "z", 
+
+  if ( !PyArg_ParseTuple( args, "z",
                           &name ) ) // name of printer
    {
        goto abort;
@@ -650,20 +655,20 @@ PyObject * setDefaultPrinter( PyObject * self, PyObject * args )
   */
 
   response = cupsDoRequest(http, request, "/admin/");
-  
+
   if ((response != NULL) && (response->request.status.status_code <= IPP_OK_CONFLICT ) )
   {
     r = 1;
   }
 
 abort:
-    
+
     if (http!=NULL)
         httpClose(http);
     if (response!=NULL)
         ippDelete(response);
     return Py_BuildValue( "i", r );
-   
+
 
 }
 
@@ -686,14 +691,14 @@ typedef struct
 
 static void job_dealloc( job_Object * self )
 {
-    
+
     Py_XDECREF(self->dest);
     Py_XDECREF(self->title);
     Py_XDECREF(self->user);
     PyObject_DEL( self );
 }
 
-static PyMemberDef job_members[] = 
+static PyMemberDef job_members[] =
 {
     { "id", T_INT, offsetof( job_Object, id ), 0, "Id" },
     { "dest", T_OBJECT_EX, offsetof( job_Object, dest ), 0, "Destination" },
@@ -706,12 +711,12 @@ static PyMemberDef job_members[] =
 
 
 
-static PyTypeObject job_Type = 
+static PyTypeObject job_Type =
 {
-    PyObject_HEAD_INIT( &PyType_Type ) 
+    PyObject_HEAD_INIT( &PyType_Type )
     0,                             /* ob_size */
     "Job",                         /* tp_name */
-    sizeof( job_Object ),          /* tp_basicsize */    
+    sizeof( job_Object ),          /* tp_basicsize */
     0,                             /* tp_itemsize */
     (destructor)job_dealloc,       /* tp_dealloc */
     0,                             /* tp_print */
@@ -747,17 +752,17 @@ static PyTypeObject job_Type =
     0,                             /* tp_init */
     0,//(initproc)job_init,            /* tp_init */
     0,                             /* tp_alloc */
-    //PyType_GenericAlloc,  
-    0, //job_new,                       /* tp_new */    
+    //PyType_GenericAlloc,
+    0, //job_new,                       /* tp_new */
     //PyType_GenericNew,
 };
-    
+
 
 static /*job_Object **/ PyObject * _newJob( int id, int state, char * dest, char * title, char * user, int size )
 {
     job_Object * jo;
     jo = PyObject_New( job_Object, &job_Type );
-    if ( jo==NULL ) 
+    if ( jo==NULL )
         return NULL;
     jo->id = id;
     jo->size = size;
@@ -766,19 +771,19 @@ static /*job_Object **/ PyObject * _newJob( int id, int state, char * dest, char
         jo->dest = PyString_FromString( dest );
     else
         jo->dest = Py_BuildValue( "" );
-        
+
     if (title != NULL)
         jo->title = PyString_FromString( title );
     else
         jo->title = Py_BuildValue( "" );
-    
+
     if (user != NULL)
         jo->user = PyString_FromString( user );
     else
         jo->user = Py_BuildValue( "" );
-    
+
     return (PyObject *)jo;
-    
+
 }
 
 static /*job_Object **/ PyObject * newJob( PyObject * self, PyObject * args, PyObject * kwargs )
@@ -789,16 +794,16 @@ static /*job_Object **/ PyObject * newJob( PyObject * self, PyObject * args, PyO
     char * title = "";
     char * user = "";
     int size = 0;
-            
+
     char * kwds[] = { "id", "state", "dest", "title", "user", "size", NULL };
-        
-    if (!PyArg_ParseTupleAndKeywords( args, kwargs, "i|izzzi", kwds, 
+
+    if (!PyArg_ParseTupleAndKeywords( args, kwargs, "i|izzzi", kwds,
                                       &id, &state, &dest, &title, &user, &size))
-        return NULL;        
+        return NULL;
 
     return _newJob( id, state, dest, title, user, size );
-    
-}   
+
+}
 
 
 
@@ -820,14 +825,14 @@ PyObject * cancelJob( PyObject * self, PyObject * args ) // cancelJob( dest, job
     int status;
     int jobid;
     char * dest;
-    
+
     if ( !PyArg_ParseTuple( args, "si", &dest, &jobid ) )
     {
         return Py_BuildValue( "i", 0 );
     }
-    
+
     status = cupsCancelJob( dest, jobid );
-    
+
     return Py_BuildValue( "i", status );
 }
 
@@ -837,31 +842,30 @@ PyObject * getJobs( PyObject * self, PyObject * args )
     int i;
     int num_jobs;
     PyObject * job_list;
-    PyObject * newjob;
     int my_job;
     int completed;
-    
+
     if ( !PyArg_ParseTuple( args, "ii", &my_job, &completed ) )
     {
         return PyList_New( 0 );
     }
-    
+
     num_jobs = cupsGetJobs( &jobs, NULL, my_job, completed );
-    
+
     if ( num_jobs > 0 )
     {
         job_list = PyList_New( num_jobs );
-        
+
         for ( i=0; i < num_jobs; i++ )
         {
             job_Object * newjob;
-            newjob = (job_Object *)_newJob(  jobs[i].id, 
+            newjob = (job_Object *)_newJob(  jobs[i].id,
                             jobs[i].state,
-                            jobs[i].dest, 
-                            jobs[i].title, 
-                            jobs[i].user, 
+                            jobs[i].dest,
+                            jobs[i].title,
+                            jobs[i].user,
                             jobs[i].size );
-                            
+
             PyList_SetItem( job_list, i, (PyObject *)newjob );
 
         }
@@ -874,27 +878,128 @@ PyObject * getJobs( PyObject * self, PyObject * args )
     return job_list;
 }
 
-
-
-
-PyObject *  getVersion( PyObject * self, PyObject * args ) 
+PyObject *  getVersion( PyObject * self, PyObject * args )
 {
     //return Py_BuildValue( "iii", CUPS_VERSION_MAJOR, CUPS_VERSION_MINOR, CUPS_VERSION_PATCH );
     return Py_BuildValue( "f", CUPS_VERSION );
 }
 
-PyObject *  getServer( PyObject * self, PyObject * args ) 
+ppd_file_t * ppd_file = NULL;
+
+PyObject *  openPPD( PyObject * self, PyObject * args )
+{
+    char * printer;
+    const char * ppd;
+    FILE * file;
+
+    if ( !PyArg_ParseTuple( args, "z", &printer ) )
+    {
+        return Py_BuildValue( "" ); // None
+    }
+
+    ppd = cupsGetPPD( (const char *)printer );
+
+    file = fopen( ppd, "r" );
+    ppd_file = ppdOpen( file );
+    fclose(file);
+
+    ppdMarkDefaults( ppd_file );
+
+    return Py_BuildValue( "" ); // None
+}
+
+PyObject *  closePPD( PyObject * self, PyObject * args )
+{
+    if( ppd_file != NULL )
+    {
+        ppdClose( ppd_file );
+    }
+
+    ppd_file = NULL;
+
+    return Py_BuildValue( "" ); // None
+}
+
+PyObject *  getPPDOption( PyObject * self, PyObject * args )
+{
+    if( ppd_file != NULL )
+    {
+        char * option;
+
+        if ( !PyArg_ParseTuple( args, "z", &option ) )
+        {
+            return Py_BuildValue( "" ); // None
+        }
+
+        ppd_choice_t * marked_choice;
+        marked_choice = ppdFindMarkedChoice( ppd_file, option );
+
+        if (marked_choice == NULL )
+        {
+            return Py_BuildValue( "" ); // None
+        }
+        else
+        {
+            return Py_BuildValue( "s", marked_choice->text );
+        }
+    }
+    else
+    {
+        return Py_BuildValue( "" ); // None
+    }
+}
+
+PyObject *  getPPDPageSize( PyObject * self, PyObject * args )
+{
+    //PyRun_SimpleString( "print '0'" );
+
+    if( ppd_file != NULL )
+    {
+        //ppd_size_t *size;
+        float      width;
+        float      length;
+
+        ppd_choice_t * page_size;
+        page_size = ppdFindMarkedChoice( ppd_file, "PageSize" );
+
+        if( page_size == NULL )
+        {
+            PyRun_SimpleString( "print '2'" );
+            //return Py_BuildValue( "" );
+        }
+
+        //size   = ppdPageSize(ppd, page_size->text );
+        width  = ppdPageWidth(ppd_file, page_size->text );
+        length = ppdPageLength(ppd_file, page_size->text );
+        //PyRun_SimpleString( "print '3'" );
+
+        return Py_BuildValue( "(sff)", page_size->text, width, length );
+    }
+    else
+    {
+        //PyRun_SimpleString( "print '1'" );
+        return Py_BuildValue( "" );
+    }
+}
+
+
+
+PyObject *  getServer( PyObject * self, PyObject * args )
 {
     return Py_BuildValue( "s", cupsServer() );
 }
 
 
-static PyMethodDef cupsext_methods[] = 
+static PyMethodDef cupsext_methods[] =
 {
     { "getPrinters", (PyCFunction)getPrinters, METH_VARARGS },
     { "addPrinter",  (PyCFunction)addPrinter,  METH_VARARGS },
     { "delPrinter",  (PyCFunction)delPrinter,  METH_VARARGS },
     { "getDefault",  (PyCFunction)getDefault,  METH_VARARGS },
+    { "openPPD",     (PyCFunction)openPPD,     METH_VARARGS },
+    { "closePPD",    (PyCFunction)closePPD,    METH_VARARGS },
+    { "getPPDOption",(PyCFunction)getPPDOption,METH_VARARGS },
+    { "getPPDPageSize", (PyCFunction)getPPDPageSize, METH_VARARGS },
     { "getVersion",  (PyCFunction)getVersion,  METH_VARARGS },
     { "cancelJob",   (PyCFunction)cancelJob,   METH_VARARGS },
     { "getJobs",     (PyCFunction)getJobs,     METH_VARARGS },
@@ -902,22 +1007,22 @@ static PyMethodDef cupsext_methods[] =
     { "Job",         (PyCFunction)newJob,      METH_VARARGS | METH_KEYWORDS },
     { "Printer",     (PyCFunction)newPrinter,  METH_VARARGS | METH_KEYWORDS },
     { NULL, NULL }
-};  
+};
 
 
 static char cupsext_documentation[] = "Python extension for CUPS 1.x";
 
 void initcupsext( void )
 {
-    
-    PyObject * mod = Py_InitModule4( "cupsext", cupsext_methods, 
-                                     cupsext_documentation, (PyObject*)NULL, 
-                     PYTHON_API_VERSION );
-                     
+
+    PyObject * mod = Py_InitModule4( "cupsext", cupsext_methods,
+                                     cupsext_documentation, (PyObject*)NULL,
+                                     PYTHON_API_VERSION );
+
     if ( mod == NULL )
-        return;                  
-                     
-                     
+        return;
+
+
 }
 
 
