@@ -69,6 +69,8 @@ DJGenericVIP::DJGenericVIP (SystemServices* pSS, BOOL proto)
 
     pMode[ModeCount++] = new VIPFastDraftMode ();        // Fast Draft
     pMode[ModeCount++] = new VIPGrayFastDraftMode ();    // Grayscale Fast Draft
+    pMode[ModeCount++] = new VIPAutoPQMode ();           // Printer selects PrintMode
+    pMode[ModeCount++] = new VIPFastPhotoMode ();        // Fast Photo
 
     for (int i = 0; i < (int) ModeCount; i++)
     {
@@ -120,6 +122,39 @@ VIPGrayFastDraftMode::VIPGrayFastDraftMode () : GrayMode (ulMapDJ600_CCM_K)
     pmColor     = GREY_K;
 }
 
+VIPFastPhotoMode::VIPFastPhotoMode () : PrintMode (NULL)
+{
+    bFontCapable = FALSE;
+#if defined(APDK_VIP_COLORFILTERING)
+    Config.bErnie = FALSE;
+#endif
+    medium = mediaHighresPhoto;
+    theQuality = qualityFastDraft;
+    pmQuality = QUALITY_FASTDRAFT;
+    pmMediaType = MEDIA_PHOTO;
+} // VIPFastPhotoMode
+
+VIPAutoPQMode::VIPAutoPQMode () : PrintMode (NULL)
+{
+    BaseResX =
+    BaseResY = 600;
+    ResolutionX[0] = 600;
+    ResolutionY[0] = 600;
+    bFontCapable = FALSE;
+#ifdef APDK_AUTODUPLEX
+    bDuplexCapable = FALSE;
+#endif
+#if defined(APDK_VIP_COLORFILTERING)
+    Config.bErnie = TRUE;
+#endif
+
+    Config.bColorImage = FALSE;
+
+    medium     = mediaAuto;
+    theQuality = qualityAuto;
+    pmQuality  = QUALITY_AUTO;
+} // VIPAutoPQMode
+
 BOOL DJGenericVIP::UseGUIMode (PrintMode* pPrintMode)
 {
     return TRUE;
@@ -147,7 +182,7 @@ PAPER_SIZE DJGenericVIP::MandatoryPaperSize ()
  *      100 - 13 X 19 size
  */
 
-    if (IOMode.bDevID && ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, TRUE)) == NO_ERROR))
+    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -183,7 +218,7 @@ BOOL DJGenericVIP::FullBleedCapable (PAPER_SIZE ps, FullbleedType  *fbType, floa
 {
     BYTE    sDevIdStr[DevIDBuffSize];
     char    *pStr;
-    if (IOMode.bDevID && ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, TRUE)) == NO_ERROR))
+    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -296,6 +331,26 @@ BOOL DJGenericVIP::FullBleedCapable (PAPER_SIZE ps, FullbleedType  *fbType, floa
     return FALSE;
 }
 
+BOOL DJGenericVIP::GetMargins (PAPER_SIZE ps, float *fMargins)
+{
+    float           xo, yo, xl, yt;
+    FullbleedType   fbType = fullbleedNotSupported;
+
+    fMargins[0] = (float) 0.125;
+    fMargins[1] = (float) 0.125;
+    fMargins[2] = (float) 0.125;
+    fMargins[3] = (float) 0.5;
+    if ((FullBleedCapable (ps, &fbType, &xo, &yo, &xl, &yt)) &&
+        (fbType == fullbleed4EdgeAllMedia ||
+         fbType == fullbleed4EdgePhotoMedia ||
+         fbType == fullbleed4EdgeNonPhotoMedia))
+    {
+        fMargins[3] = (float) 0.125;
+    }
+
+    return TRUE;
+}
+
 
 PHOTOTRAY_STATE DJGenericVIP::PhotoTrayEngaged (BOOL bQueryPrinter)
 {
