@@ -58,6 +58,7 @@ public:
 
     virtual BOOL HagakiFeedDuplexerPresent(BOOL bQueryPrinter);
 #endif
+    virtual BOOL GetMargins (PAPER_SIZE ps, float *fMargins);
 
 private:
 	virtual void AdjustModeSettings (BOOL bDoFullBleed, MEDIATYPE ReqMedia,
@@ -77,6 +78,18 @@ public:
     VIPGrayFastDraftMode ();
 }; // VIPGrayFastDraftMode
 
+class VIPFastPhotoMode : public PrintMode
+{
+public:
+    VIPFastPhotoMode ();
+}; // VIPFastPhotoMode
+
+class VIPAutoPQMode : public PrintMode
+{
+public:
+    VIPAutoPQMode ();
+}; // VIPAutoPQMode
+
 #if defined(APDK_DJGENERICVIP) && defined (APDK_DJ9xxVIP)
 //! DJGenericVIPProxy
 /*!
@@ -88,12 +101,14 @@ public:
         "GenericVIP",                       // family name
         "dj450\0"
 		"deskjet 450\0"
+        "deskjet 5100\0"
+        "Deskjet 5400\0"
         "deskjet 5550\0"
         "deskjet 5551\0"
         "deskjet 5600\0"
-        "deskjet 5100\0"
 		"Deskjet 5700\0"
 		"deskjet 5800\0"
+        "Deskjet 5900\0"
 		"Deskjet 6500\0"
 		"Deskjet 6600\0"
 		"Deskjet 6800\0"
@@ -111,16 +126,26 @@ public:
         "photosmart 7260\0"
 		"photosmart 7268\0"
 		"Photosmart 7400\0"
+        "Photosmart 7800\0"
+        "Photosmart 8000\0"
 		"Photosmart 8100\0"
 		"Photosmart 8400\0"
-		"Photosmart 8750\0"
+		"Photosmart 8700\0"
+		"Photosmart 8200\0"
+        "Business Inkjet 1000\0"
+		"Photosmart 320\0"
+		"Photosmart 370\0"
+		"Photosmart 380\0"
+		"Photosmart 330\0"
+        "Photosmart 420\0"
 #ifdef APDK_MLC_PRINTER
-		"psc 1600\0"
+		"PSC 1500\0"
+		"PSC 1600\0"
         "PSC 2100\0"              
         "PSC 2150\0"
         "PSC 2200\0"
         "psc 2300\0"
-		"psc 2350\0"
+		"PSC 2350\0"
         "psc 2400\0"
         "psc 2500\0"
 		"PSC 2170\0"
@@ -130,8 +155,12 @@ public:
 		"Officejet 7300\0"
 		"Officejet 7200\0"
 		"Officejet 6200\0"
+        "Photosmart 2570\0"
 		"Photosmart 2600\0"
 		"Photosmart 2700\0"
+        "Photosmart 3100\0"
+        "Photosmart 3200\0"
+        "Photosmart 3300\0"
 #endif
     ) {m_iPrinterType = eDJGenericVIP;}
     inline Printer* CreatePrinter(SystemServices* pSS) const { return new DJGenericVIP(pSS); }
