@@ -382,6 +382,7 @@ class LJFastRasterProxy;
 
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
 class PSP100Proxy;
+class PSP470Proxy;
 #endif
 
 #if defined(APDK_DJGENERICVIP) && defined (APDK_DJ9xxVIP)
@@ -520,6 +521,7 @@ public:
 
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
     static PSP100Proxy s_PSP100Proxy;
+    static PSP470Proxy s_PSP470Proxy;
 #endif
 
 #if defined(APDK_DJGENERICVIP) && defined (APDK_DJ9xxVIP)
@@ -656,11 +658,12 @@ public:
 class Mode9 : public Compressor
 {
 public:
-    Mode9(SystemServices* pSys,unsigned int RasterSize);
+    Mode9(SystemServices* pSys,unsigned int RasterSize, BOOL bVIPPrinter = FALSE);
     virtual ~Mode9();
     BOOL Process(RASTERDATA* input);
     void Flush();
 	BOOL ResetSeedRow;
+    BOOL m_bVIPPrinter;
 }; //Mode9
 
 //ClassName
@@ -780,6 +783,7 @@ friend class ModeJPEG;
 
 public:
     Header(Printer* p,PrintContext* pc);
+    virtual ~Header ();
 
     virtual DRIVER_ERROR Send()=0;
 
