@@ -60,6 +60,7 @@ typedef enum PRINTER_TYPE               // typedef'ed for C interface
     eDJ350,
     eDJ8x5,
     ePSP100,
+    ePSP470,
     eDJ3320,
     eDJGenericVIP,
 	eLJMono,
@@ -94,6 +95,7 @@ const char ModelName[MAX_PRINTER_TYPE][13] =
     "DJ350",
     "DJ8x5",
     "PS100",
+    "PS470",
     "DJ3320",
     "GenericVIP",
 	"Mono Laser",
